package com.example.authapi.security.controller;

import com.example.authapi.security.dto.MessageDto;
import com.example.authapi.security.dto.TokenDto;
import com.example.authapi.security.jwt.JwtProvider;
import com.example.authapi.security.service.UserEntityService;
import com.example.authapi.users.application.dto.UserDTO;
import com.example.authapi.users.application.dto.UserNoExtraDataDTO;
import com.example.authapi.users.application.email.EmailService;
import com.example.authapi.users.infrastructure.entity.UserEntity;
import com.example.authapi.users.infrastructure.mapper.UserMapper;
import com.example.authapi.users.infrastructure.repository_outputAdapter.SqlUserRepositoryImp;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;


@RestController
@RequestMapping("/auth")
@CrossOrigin//(origins = "http://localhost:9000/")
@Slf4j
public class AuthController implements AuthInputPort{

    @Autowired
    UserEntityService userEntityService;

    @Autowired
    EmailService email;

    @Autowired
    JwtProvider jwtProvider;

    @Autowired
    SqlUserRepositoryImp userService;

    //@Value("${jwt.accessTokenCookieName}")
    //private String cookieName;

    @Override
    @PostMapping("/create")
    public ResponseEntity<MessageDto> create(@Valid @RequestBody UserDTO dto) throws Exception {
        log.info("create...");
        if (dto.getRepeatPassword()!=null){
            if (!dto.getRepeatPassword().equals(dto.getPassword())){
                log.info("...create1");
                return ResponseEntity.badRequest().build();
            }
        }
        UserEntity userEntity = userEntityService.create(dto);
        // Enviar email para su verificación
        TokenDto tokenDto = userEntityService.login(dto, false);
        String url = "http://localhost:5000";
        String cuerpo="<h1>Para verificar el correo facilitado</h1>\n<p>Si te has dado de alta en " +
                "la web de Deep Knowledge...</p>\n"
                +"<p>Has de pinchar en el siguiente enlace: <a href='"+url+"/auth/verificar?clave="+
                tokenDto.getToken()+ "'>verificar</a></p>";

        email.sendHtmlMessage(userEntity.getEmail(),"Correo de verificación",cuerpo);
        log.info("...create2");
        return ResponseEntity.ok(new MessageDto(HttpStatus.OK, "user " + userEntity.getUsername() + " have been created"));
    }

    @Override
    @PostMapping("/login")
    public ResponseEntity<TokenDto> login(HttpServletResponse httpServletResponse, @RequestBody UserDTO dto) throws Exception {
        log.info("login...");
        TokenDto jwtTokenDtoDto = userEntityService.login(dto,true);
        log.info("...login");
        return ResponseEntity.ok(jwtTokenDtoDto);
    }

    @Override
    @GetMapping("/verificar")
    public ResponseEntity<String> emailVerificado(@RequestParam(name = "clave") String token, HttpServletResponse httpServletResponse ){
        log.info("emailVerificado...");
        //System.out.println(token);
        if (jwtProvider.validateToken(token)) {
            String username=jwtProvider.getUserNameFromToken(token);
            UserDTO userDTO =userService.findByUsername(username);
            userDTO.setIsEmailVerify(true);
            UserDTO userSaved=userService.create(userDTO);
            //System.out.println("Usuario actualizado el campo isEmailVerify "+userSaved);
            log.info("...emailVerificado1");
            return new ResponseEntity<>(("Email Verificado"), HttpStatus.OK) ;
        }
        log.info("...emailVerificado2");
        return new ResponseEntity<>(("Email No Verificado"), HttpStatus.BAD_REQUEST) ;
    }

    @Override
    @GetMapping("/logout")
    public ResponseEntity<String> logOut(HttpServletResponse httpServletResponse){
        log.info("...logOut...");
        //CookieUtil.clear(httpServletResponse,cookieName);
        return new ResponseEntity<>(("Sesión cerrada"), HttpStatus.OK) ;
    }

    @Override
    @GetMapping("/tokentouser/{token}")
    public UserNoExtraDataDTO tokenToUser (@PathVariable("token") String token) {
        log.info("tokenToUser...");
        if (jwtProvider.validateToken(token)) {
            String username = jwtProvider.getUserNameFromToken(token);
            log.info("...tokenToUser1");
            UserDTO userDTO = userService.findByUsername(username);
            return UserMapper.getInstance().dtoToDtoNoExtraData(userDTO);
        }
        log.info("...tokenToUser2");
        throw new RuntimeException("Token caducado o invalido.\nEs necesario logearse");
    }

}
