package com.example.authapi.security.service;


import com.example.authapi.users.application.dto.RolDTO;
import com.example.authapi.users.application.dto.UserDTO;
import com.example.authapi.users.application.repository.UserRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Set;

@Service
@Slf4j
public class UserDetailsServiceImpl implements UserDetailsService {

    @Autowired
    UserRepository userRepository;
    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        log.info("loadUserByUsername...");
        UserDTO userDTO = userRepository.findByUsernameOrEmail(username, username);
        //UserDomain userDomain = UserMapperApplication.getInstance().dtoToDomain(userDTO);

        if (userDTO == null) {
            log.info("...loadUserByUsername1");
            throw new UsernameNotFoundException("User not found");
        }
        Set<RolDTO> auths = userDTO.getRoles();
        Collection<GrantedAuthority> authorities = new ArrayList<>();
        for (RolDTO auth : auths) {
            authorities.add(new SimpleGrantedAuthority("ROLE_" + auth.getName()));
        }
        userDTO.setAuthorities(authorities);
        log.info("...loadUserByUsername2");
        return userDTO;
    }
}
