package com.example.authapi.security.service;

import com.example.authapi.security.dto.TokenDto;
import com.example.authapi.security.jwt.JwtProvider;
import com.example.authapi.users.application.dto.RolDTO;
import com.example.authapi.users.application.dto.UserDTO;
import com.example.authapi.users.application.repository.UserRepository;
import com.example.authapi.users.infrastructure.entity.UserEntity;
import com.example.authapi.users.infrastructure.mapper.UserMapper;
import com.example.authapi.users.infrastructure.repository_outputAdapter.SqlRolRepositoryImp;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.Set;

@Service
@Slf4j
public class UserEntityService {

    @Autowired
    UserRepository userRepository;

    @Autowired
    PasswordEncoder passwordEncoder;

    @Autowired
    JwtProvider jwtProvider;

    @Autowired
    AuthenticationManager authenticationManager;

    @Autowired
    SqlRolRepositoryImp sqlRolRepository;


    public UserEntity create(UserDTO dto) throws Exception {
        log.info("create...");
        if(userRepository.findByUsernameOrEmail(dto.getUsername(), dto.getEmail())!=null){
            log.info("...create1");
            throw new Exception("username already in use");
        }
        if (dto.getRoles().isEmpty()){
            RolDTO rolDTO = sqlRolRepository.getByName("User");
            Set<RolDTO> roles= new HashSet<>();
            roles.add(rolDTO);
            dto.setRoles(roles);
        }
        log.info("...create2");
        return UserMapper.getInstance().dtoToEntity(userRepository.create(dto));
    }


    public TokenDto login(UserDTO dto, boolean veri) {
        log.info("login...");
        /*
        Authentication authentication =
                authenticationManager.authenticate(new UsernamePasswordAuthenticationToken(dto.getUsername(), dto.getPassword()));
        SecurityContextHolder.getContext().setAuthentication(authentication);
        String token = jwtProvider.createToken(dto);
        return new Token(token);
         */
        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(dto.getEmail()==null?dto.getUsername():dto.getEmail(),dto.getPassword());
        Authentication authentication = authenticationManager.authenticate(authenticationToken);
        UserDTO aut=UserMapper.getInstance().objectAuthToDto(authentication.getPrincipal());
        if (veri && !aut.getIsEmailVerify()){
            log.info("...login1");
            return null;
        }
        SecurityContextHolder.getContext().setAuthentication(authentication);
        String token = jwtProvider.createToken((UserDTO) authentication.getPrincipal());
//        String token = jwtProvider.createToken(dto);
        log.info("...login2");
        return new TokenDto(token);
    }

}

