package com.example.authapi.shared.exceptions;

import com.example.authapi.shared.valueObject.I18nMessage;
import lombok.extern.slf4j.Slf4j;

import java.util.*;

@Slf4j
public class TfgException extends RuntimeException {
    private final List<I18nMessage> errorMessages = new ArrayList();

    public TfgException(){
        log.error("...TfgException...");
    }

    public TfgException(String key, Object... params){
        super(key);
        log.error("TfgException...");
        this.addError(key,params);
        log.error("...TfgException");
    }

    public TfgException(Throwable t, String key, Object... params){
        super(key, t);
        log.error("TfgException...");
        this.addError(key,params);
        log.error("...TfgException");
    }

    /** @deprecated **/
    @Deprecated
    public Map<String, List<Object>> getErrorMap(){
        log.error("getErrorMap...");
        Map<String, List<Object>> map = new HashMap<>();

        for (I18nMessage msg : this.errorMessages) {
            map.put(msg.getKey(), msg.getParams());
        }
        log.error("...getErrorMap");
        return map;
    }

    public void addErrors(Map<String, List<?>> errorMap){
        log.error("addErrors...");
        Iterator iterator = errorMap.entrySet().iterator();

        while (iterator.hasNext()){
            Map.Entry<String,List<?>>  e = (Map.Entry) iterator.next();
            this.addError((String) e.getKey(), e.getValue());
        }
        log.error("...addErrors");
    }

    public void addError(String key, Object... params){
        log.error("addErrors...");
        this.errorMessages.add(new I18nMessage(key, Arrays.asList(params)));
        log.error("...addErrors");
    }

    public List<I18nMessage> getErrorMessages(){
        log.error("...getErrorMessages...");
        return this.errorMessages;
    }

    public Boolean containsError(String key){
        log.error("...containsError...");
        return this.errorMessages.contains(new I18nMessage(key));
    }
}
