package com.example.authapi.shared.valueObject;


import lombok.*;
import lombok.extern.slf4j.Slf4j;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@Builder
@Slf4j
public class TfgDTO<T> implements Serializable {
    protected List<I18nMessage> errorMessages = new ArrayList<>();
    protected List<I18nMessage> warningMessages = new ArrayList<>();
    protected List<I18nMessage> infoMessages = new ArrayList<>();

    public void addInfo(String key, Object... params) {
        log.info("addInfo...");
        this.infoMessages.add(new I18nMessage(key, Arrays.asList(params)));
        log.info("...addInfo");
    }

    public void addError(String key, Object... params) {
        log.info("addError...");
        this.errorMessages.add(new I18nMessage(key, Arrays.asList(params)));
        log.info("...addError");
    }

    public void addWarning(String key, Object... params) {
        log.info("addWarning...");
        this.warningMessages.add(new I18nMessage(key, Arrays.asList(params)));
        log.info("...addWarning");
    }

    public boolean hasErrors(){
        log.info("...hasErrors...");
        return !this.errorMessages.isEmpty();
    }



    public boolean hasObjectNotFoundException(){
        log.info("...hasObjectNotFoundException...");
        return this.getErrorMessages().stream().anyMatch((em) -> {
            return em.getKey().equalsIgnoreCase("general.objectnotfound");
        });
    }

}
