package com.example.authapi.users.application.mapper;

import com.example.authapi.users.application.dto.AvatarDTO;
import com.example.authapi.users.domain.entity.AvatarDomain;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class AvatarMapperApplication implements Serializable {
    private static AvatarMapperApplication instance;

    private AvatarMapperApplication(){}

    public static AvatarMapperApplication getInstance(){
        if(instance==null){
            instance = new AvatarMapperApplication();
        }
        return instance;
    }


    public AvatarDTO domainToDto(AvatarDomain avatarDomain) {
        if(avatarDomain == null){
            return null;
        }
        return AvatarDTO.builder()
                .id(avatarDomain.getId())
                .userId(avatarDomain.getUserDomain())
                .path(avatarDomain.getPath())
                .url(avatarDomain.getUrl())
                .type(avatarDomain.getType())
                .build();
    }

    public AvatarDomain dtoToDomain(AvatarDTO avatarDTO) {
        if(avatarDTO == null){
            return null;
        }
        return AvatarDomain.builder()
                .id(avatarDTO.getId())
                .userDomain(avatarDTO.getUserId())
                .path(avatarDTO.getPath())
                .url(avatarDTO.getUrl())
                .type(avatarDTO.getType())
                .build();
    }

    public List<AvatarDomain> listDtoToDomainList(List<AvatarDTO> avatarDTOList){
        if(avatarDTOList==null){
            return Collections.emptyList();
        }
        return avatarDTOList.stream().map(this::dtoToDomain).collect(Collectors.toList());
    }

    public List<AvatarDTO> listDomainToDtoList(List<AvatarDomain> avatarDomainList){
        if(avatarDomainList==null){
            return Collections.emptyList();
        }
        return avatarDomainList.stream().map(this::domainToDto).collect(Collectors.toList());
    }

}
