package com.example.authapi.users.application.mapper;


import com.example.authapi.users.application.dto.RolDTO;
import com.example.authapi.users.application.dto.UserDTO;
import com.example.authapi.users.domain.entity.RolDomain;
import com.example.authapi.users.domain.entity.UserDomain;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class UserMapperApplication implements Serializable {
    private static UserMapperApplication instance;

    private UserMapperApplication(){}

    public static UserMapperApplication getInstance(){
        if(instance==null){
            instance = new UserMapperApplication();
        }
        return instance;
    }

    public UserDTO domainToDto(UserDomain userDomain) {
        if(userDomain == null){
            return null;
        }
        return UserDTO.builder()
                .id(userDomain.getId())
                .email(userDomain.getEmail())
                .username(userDomain.getUsername())
                .url(userDomain.getUrl())
                .roles(userDomain.getRoles().stream().map(rolEntity -> new RolDTO(rolEntity.getId(), rolEntity.getName())).collect(Collectors.toSet()))
                .isEmailVerify(userDomain.getIsEmailVerify())
                .build();
    }

    public UserDomain dtoToDomain(UserDTO userDTO){
        if(userDTO == null){
            return null;
        }
        return UserDomain.builder()
                .id(userDTO.getId())
                .email(userDTO.getEmail())
                .username(userDTO.getUsername())
                .url(userDTO.getUrl())
                .roles(userDTO.getRoles().stream().map(rolEntity -> new RolDomain(rolEntity.getId(), rolEntity.getName())).collect(Collectors.toSet()))
                .isEmailVerify(userDTO.getIsEmailVerify())
                .build();
    }

    public List<UserDTO> listDomainToListDto(List<UserDomain> userDomainList){
        if(userDomainList==null){
            return Collections.emptyList();
        }
        return userDomainList.stream().map(this::domainToDto).collect(Collectors.toList());
    }

    public List<UserDomain> listDtoToDomainList(List<UserDTO> userDTOList){
        if(userDTOList ==null){
            return Collections.emptyList();
        }
        return userDTOList.stream().map(this::dtoToDomain).collect(Collectors.toList());
    }


}


