package com.example.authapi.users.application.mapper;


import com.example.authapi.users.application.dto.UserSettingsDTO;
import com.example.authapi.users.domain.entity.UserSettingsDomain;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class UserSettingsMapperApplication implements Serializable {
    private static UserSettingsMapperApplication instance;

    private UserSettingsMapperApplication(){}

    public static UserSettingsMapperApplication getInstance(){
        if(instance==null){
            instance = new UserSettingsMapperApplication();
        }
        return instance;
    }

    public UserSettingsDTO domainToDto(UserSettingsDomain userSettingsDomain) {
        if(userSettingsDomain == null){
            return null;
        }
        return UserSettingsDTO.builder()
                .id(userSettingsDomain.getId())
                .languageEnum(userSettingsDomain.getLanguageEnum())
                .visualizationTypeEnum(userSettingsDomain.getVisualizationTypeEnum())
                .build();
    }

    public UserSettingsDomain dtoToDomain(UserSettingsDTO userSettingsDTO){
        if(userSettingsDTO == null){
            return null;
        }
        return UserSettingsDomain.builder()
                .id(userSettingsDTO.getId())
                .languageEnum(userSettingsDTO.getLanguageEnum())
                .visualizationTypeEnum(userSettingsDTO.getVisualizationTypeEnum())
                .build();
    }

    public List<UserSettingsDTO> listDomainToListDto(List<UserSettingsDomain> userSettingsDomainList){
        if(userSettingsDomainList==null){
            return Collections.emptyList();
        }
        return userSettingsDomainList.stream().map(this::domainToDto).collect(Collectors.toList());
    }

    public List<UserSettingsDomain> listDtoToDomainList(List<UserSettingsDTO> userSettingsDTOList){
        if(userSettingsDTOList ==null){
            return Collections.emptyList();
        }
        return userSettingsDTOList.stream().map(this::dtoToDomain).collect(Collectors.toList());
    }

}


