package com.example.authapi.users.application.user.update;

import com.example.authapi.users.application.dto.UserDTO;
import com.example.authapi.users.application.mapper.UserMapperApplication;
import com.example.authapi.users.application.repository.UserRepository;
import com.example.authapi.users.domain.entity.UserDomain;
import com.example.authapi.users.domain.exceptions.ExceptionUserNotExistException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

@Service
public class UpdateUserImp implements UpdateUser {

    private final UserRepository userRepository;
    private final PasswordEncoder passwordEncoder;

    @Autowired
    public UpdateUserImp(UserRepository userRepository,PasswordEncoder passwordEncoder) {
        this.userRepository = userRepository;
        this.passwordEncoder=passwordEncoder;
    }

    public UserDTO rename(Integer id, String username) {
        UserDTO userDTO = userRepository.getById(id);
        UserDomain userDomain = UserMapperApplication.getInstance().dtoToDomain(userDTO);

        if(userDTO == null){
            throw new ExceptionUserNotExistException();
        }
        userDomain.rename(username);
        userDTO = UserMapperApplication.getInstance().domainToDto(userDomain);

        userRepository.update(userDTO);
        return userDTO;
    }


    public UserDTO update(UserDTO userDTO) {
        UserDTO userDto = userRepository.getById(userDTO.getId());
        if (userDTO.getPassword()!=null)
            userDto.setPassword(userDTO.getPassword());
        if (userDTO.getEmail()!=null)
            userDto.setEmail(userDTO.getEmail());
        if (userDTO.getUsername()!=null)
            userDto.setUsername(userDTO.getUsername());
        if (userDTO.getRoles()!=null && !userDTO.getRoles().isEmpty())
            userDto.setRoles(userDTO.getRoles());
        if (userDTO.getIsEmailVerify()!=null)
            userDto.setIsEmailVerify(userDTO.getIsEmailVerify());
        if (userDTO.getAvatarDTO()!=null)
            userDto.setAvatarDTO(userDTO.getAvatarDTO());
        return userRepository.update(userDto);
    }
}
