package com.example.authapi.users.application.userSettings.get;

import com.example.authapi.shared.enums.LanguageEnum;
import com.example.authapi.shared.enums.VisualizationTypeEnum;
import com.example.authapi.users.application.dto.UserSettingsDTO;
import com.example.authapi.users.application.repository.UserSettingsRepository;
import com.example.authapi.users.domain.exceptions.ExceptionUserSettingsNotExistException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class GetUserSettingsByIdImp implements GetUserSettingsById{
    private final UserSettingsRepository userSettingsRepository;

    @Autowired
    public GetUserSettingsByIdImp(UserSettingsRepository userSettingsRepository) {
        this.userSettingsRepository=userSettingsRepository;
    }


    @Override
    public UserSettingsDTO getByID(Integer id) {
        UserSettingsDTO userSettingsDTO = userSettingsRepository.getUserSettingsById(id);

        if(userSettingsDTO ==null){
            throw new ExceptionUserSettingsNotExistException();
        }

        return new UserSettingsDTO(
                        userSettingsDTO.getId(),
                        userSettingsDTO.getLanguageEnum(),
                        userSettingsDTO.getVisualizationTypeEnum()
        );
    }

    public List<UserSettingsDTO> getAllUserSettings() {
        return userSettingsRepository.getAllUserSettings();
    }


    @Override
    public List<UserSettingsDTO> getAllByLanguageEnum(LanguageEnum languageEnum) {
        return userSettingsRepository.getAllUserSettings();
    }

    @Override
    public List<UserSettingsDTO> getAllByVisualizationTypeEnum(VisualizationTypeEnum visualizationTypeEnum) {
        return userSettingsRepository.getAllUserSettings();
    }
}
