package com.example.authapi.users.infrastructure.controller_inputAdapter;

import com.example.authapi.shared.exceptions.TfgException;
import com.example.authapi.shared.valueObject.ResponseDTO;
import com.example.authapi.users.application.avatar.create.CreateAvatarImp;
import com.example.authapi.users.application.avatar.delete.DeleteAvatarImp;
import com.example.authapi.users.application.avatar.find.FindAvatarImp;
import com.example.authapi.users.application.avatar.get.GetAvatarByIdImp;
import com.example.authapi.users.application.avatar.update.UpdateAvatarImp;
import com.example.authapi.users.application.dto.AvatarDTO;
import com.example.authapi.users.infrastructure.controller_inputPort.AvatarInputPort;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@RestController
@RequestMapping("/avatar")
@CrossOrigin
public class AvatarController implements AvatarInputPort {
    private final CreateAvatarImp createAvatarImp;
    private final DeleteAvatarImp deleteAvatarImp;
    private final FindAvatarImp findAvatarImp;
    private final GetAvatarByIdImp getAvatarByIdImp;
    private final UpdateAvatarImp updateAvatarImp;


    @Autowired
    public AvatarController(CreateAvatarImp createAvatarImp,
                            DeleteAvatarImp deleteAvatarImp,
                            FindAvatarImp findAvatarImp,
                            GetAvatarByIdImp getAvatarByIdImp,
                            UpdateAvatarImp updateAvatarImp) {
        this.createAvatarImp = createAvatarImp;
        this.deleteAvatarImp = deleteAvatarImp;
        this.findAvatarImp = findAvatarImp;
        this.getAvatarByIdImp = getAvatarByIdImp;
        this.updateAvatarImp = updateAvatarImp;
    }

    @Override
    @GetMapping("/getAll")
    public ResponseDTO<List<AvatarDTO>> getAll(){
        ResponseDTO<List<AvatarDTO>> responseDTO = new ResponseDTO<>();
        try {
            List<AvatarDTO> avatarDTOList = this.getAvatarByIdImp.getAllAvatar();
            responseDTO.setData(avatarDTOList);
            responseDTO.addInfo("GET_ALL_AVATAR_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @GetMapping("/get/{avatarId}")
    public ResponseDTO<AvatarDTO> getAvatarById(@PathVariable("avatarId") Integer avatarId){
        ResponseDTO<AvatarDTO> responseDTO = new ResponseDTO<>();
        try {
            AvatarDTO avatarDTO = this.getAvatarByIdImp.getAvatarByID(avatarId);
            responseDTO.setData(avatarDTO);
            responseDTO.addInfo("GET_ALL_AVATAR_BY_ID_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @PostMapping("/create")
    public ResponseDTO<AvatarDTO> create(@RequestBody AvatarDTO avatarDTO) {
        ResponseDTO<AvatarDTO> responseDTO = new ResponseDTO<>();
        try {
            responseDTO.setData(this.createAvatarImp.create(avatarDTO));
            responseDTO.addInfo("GET_CREATE_AVATAR_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @DeleteMapping("/delete/{avatarId}")
    public ResponseDTO<AvatarDTO> delete(@PathVariable("avatarId") Integer avatarId) {
        ResponseDTO<AvatarDTO> responseDTO = new ResponseDTO<>();
        try {
            AvatarDTO avatarDTO= this.deleteAvatarImp.delete(avatarId);
            responseDTO.setData((avatarDTO));
            responseDTO.addInfo("GET_DELETE_AVATAR_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }
}
