package com.example.authapi.users.infrastructure.controller_inputAdapter;

import com.example.authapi.shared.exceptions.TfgException;
import com.example.authapi.shared.valueObject.ResponseDTO;
import com.example.authapi.users.application.rol.create.CreateRolByIdImp;
import com.example.authapi.users.application.dto.RolDTO;
import com.example.authapi.users.application.rol.get.GetRolByIdImp;
import com.example.authapi.users.infrastructure.body.CreateRolRequestBody;
import com.example.authapi.users.infrastructure.controller_inputPort.RolInputPort;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@RestController
@RequestMapping("/rol")
@CrossOrigin
public class RolController implements RolInputPort {
    private final CreateRolByIdImp createRolByIdImp;
    private final GetRolByIdImp getRolByIdImp;

    @Autowired
    public RolController(GetRolByIdImp getRolByIdImp, CreateRolByIdImp createRolByIdImp) {
        this.getRolByIdImp = getRolByIdImp;
        this.createRolByIdImp = createRolByIdImp;
    }



    @Override
    @PreAuthorize("hasRole('ROLE_Admin')")
    @GetMapping("/get/{rolId}")
    public ResponseDTO<RolDTO> getRolById(@PathVariable("rolId") Integer rolId){
        ResponseDTO<RolDTO> responseDTO = new ResponseDTO<>();
        try {
            responseDTO.setData(this.getRolByIdImp.getRolByID(rolId));
            responseDTO.addInfo("GET_ROL_ID_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @PreAuthorize("hasRole('ROLE_Admin')")
    @GetMapping("/getAll")
    public ResponseDTO<List<RolDTO>> getAll(){
        ResponseDTO<List<RolDTO>> responseDTO = new ResponseDTO<>();
        try {
            responseDTO.setData(this.getRolByIdImp.getAllRol());
            responseDTO.addInfo("GET_ALL_ROL_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @PreAuthorize("hasRole('ROLE_Admin')")
    @PostMapping("/create")
    public ResponseDTO<RolDTO> createRol(@RequestBody CreateRolRequestBody createRolRequestBody){
        ResponseDTO<RolDTO> responseDTO = new ResponseDTO<>();
        try {
            RolDTO rolDTO =new RolDTO();
            rolDTO.setName(createRolRequestBody.getName());
            responseDTO.setData(createRolByIdImp.create(rolDTO));
            responseDTO.addInfo("CREATE_ROL_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

}
