package com.example.authapi.users.infrastructure.controller_inputAdapter;

import com.example.authapi.shared.exceptions.TfgException;
import com.example.authapi.shared.valueObject.ResponseDTO;
import com.example.authapi.users.application.avatar.get.GetAvatarById;
import com.example.authapi.users.application.user.create.CreateUserImp;
import com.example.authapi.users.application.user.delete.DeleteUserImp;
import com.example.authapi.users.application.dto.AvatarDTO;
import com.example.authapi.users.application.dto.UserDTO;
import com.example.authapi.users.application.dto.UserNoExtraDataDTO;
import com.example.authapi.users.application.user.find.FindUserImp;
import com.example.authapi.users.application.user.get.GetUserByIdImp;
import com.example.authapi.users.application.user.update.UpdateUserImp;
import com.example.authapi.users.infrastructure.body.CreateUserRequestBody;
import com.example.authapi.users.infrastructure.body.RenameUserRequestBody;
import com.example.authapi.users.infrastructure.controller_inputPort.UserInputPort;
import com.example.authapi.users.infrastructure.mapper.UserMapper;
import com.example.authapi.users.infrastructure.repository_outputAdapter.SqlUserRepositoryImp;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@RestController
@RequestMapping("/user")
@CrossOrigin
public class UserController implements UserInputPort {
    private CreateUserImp createUserImp;
    private DeleteUserImp deleteUserImp;
    private FindUserImp findUserImp;

    private GetUserByIdImp getUserByIdImp;
    private UpdateUserImp updateUserImp;

    private SqlUserRepositoryImp repositoryImp;

    private GetAvatarById getAvatarById;

    @Autowired
    public UserController(CreateUserImp createUserImp,
                          DeleteUserImp deleteUserImp,
                          FindUserImp findUserImp,
                          GetUserByIdImp getUserByIdImp,
                          UpdateUserImp updateUserImp,
                          SqlUserRepositoryImp repositoryImp,
                          GetAvatarById getAvatarById) {
        this.createUserImp = createUserImp;
        this.deleteUserImp = deleteUserImp;
        this.findUserImp = findUserImp;
        this.getUserByIdImp = getUserByIdImp;
        this.updateUserImp = updateUserImp;
        this.repositoryImp = repositoryImp;
        this.getAvatarById = getAvatarById;
    }


    @Override
    @PostMapping("/create")
    public ResponseDTO<UserNoExtraDataDTO> createUser(@RequestBody CreateUserRequestBody createUserRequestBody){
        ResponseDTO<UserNoExtraDataDTO> responseDTO = new ResponseDTO<>();
        try {
            UserDTO userDTO = this.createUserImp.create(
                    createUserRequestBody.getEmail(),
                    createUserRequestBody.getUsername()
            );
            UserNoExtraDataDTO userNoExtraDataDTO = UserMapper.getInstance().dtoToDtoNoExtraData(userDTO);
            responseDTO.setData(userNoExtraDataDTO);
            responseDTO.addInfo("USER_CREATED_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @GetMapping("/find/email/{userEmail}")
    public ResponseDTO<UserNoExtraDataDTO> findUserByEmail(@PathVariable("userEmail") String userEmail){
        ResponseDTO<UserNoExtraDataDTO> responseDTO = new ResponseDTO<>();
        try {
            UserDTO userDTO = this.findUserImp.findByEmail(userEmail);
            UserNoExtraDataDTO userNoExtraDataDTO = UserMapper.getInstance().dtoToDtoNoExtraData(userDTO);
            responseDTO.setData(userNoExtraDataDTO);
            responseDTO.addInfo("FIND_USER_EMAIL_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @DeleteMapping("/delete/{userId}")
    public ResponseDTO<UserNoExtraDataDTO> deleteUser(@PathVariable("userId") Integer userId){
        ResponseDTO<UserNoExtraDataDTO> responseDTO = new ResponseDTO<>();
        try {
            UserDTO userDTO = this.deleteUserImp.delete(userId);
            UserNoExtraDataDTO userNoExtraDataDTO = UserMapper.getInstance().dtoToDtoNoExtraData(userDTO);
            responseDTO.setData(userNoExtraDataDTO);
            responseDTO.addInfo("GET_DELETE_ID_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @GetMapping("/get/{userId}")
    public ResponseDTO<UserNoExtraDataDTO> getUserById(@PathVariable("userId") Integer userId){
        ResponseDTO<UserNoExtraDataDTO> responseDTO = new ResponseDTO<>();
        try {
            UserDTO userDTO = this.getUserByIdImp.getUserByID(userId);
            UserNoExtraDataDTO userNoExtraDataDTO = UserMapper.getInstance().dtoToDtoNoExtraData(userDTO);
            responseDTO.setData(userNoExtraDataDTO);
            responseDTO.addInfo("GET_USER_ID_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @GetMapping("/getAll")
    public ResponseDTO<List<UserNoExtraDataDTO>> getAll(){
        ResponseDTO<List<UserNoExtraDataDTO>> responseDTO = new ResponseDTO<>();
        try {
            List<UserNoExtraDataDTO> userNoExtraDataDTOList = UserMapper.getInstance().listEntityToNoExtraDataDtoList(this.repositoryImp.getAll());
            responseDTO.setData(userNoExtraDataDTOList);
            responseDTO.addInfo("GET_ALL_USER_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @PostMapping("/rename")
    public ResponseDTO<UserNoExtraDataDTO> renameUser(@RequestBody RenameUserRequestBody renameUserRequestBody){
        ResponseDTO<UserNoExtraDataDTO> responseDTO = new ResponseDTO<>();
        try {
            UserDTO userDTO =this.updateUserImp.rename(
                    renameUserRequestBody.getId(),
                    renameUserRequestBody.getName()
            );
            UserNoExtraDataDTO userNoExtraDataDTO = UserMapper.getInstance().dtoToDtoNoExtraData(userDTO);
            responseDTO.setData(userNoExtraDataDTO);
            responseDTO.addInfo("GET_RENAMED_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @PutMapping("/update")
    public ResponseDTO<UserNoExtraDataDTO> updateUser(@RequestBody UserDTO userDto){
        ResponseDTO<UserNoExtraDataDTO> responseDTO = new ResponseDTO<>();
        try {
            UserDTO userDTO = this.updateUserImp.update(userDto);
            UserNoExtraDataDTO userNoExtraDataDTO = UserMapper.getInstance().dtoToDtoNoExtraData(userDTO);
            responseDTO.setData(userNoExtraDataDTO);
            responseDTO.addInfo("USER_UPDATE_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @GetMapping("/avatar/{userId}/{avatarId}")
    public ResponseDTO<UserNoExtraDataDTO> asignarAvatarToUser(@PathVariable("userId") Integer userId,@PathVariable("avatarId") Integer avatarId) {
        ResponseDTO<UserNoExtraDataDTO> responseDTO = new ResponseDTO<>();
        try {
            UserDTO userDTO= this.getUserByIdImp.getUserByID(userId);
            AvatarDTO avatarDTO=this.getAvatarById.getAvatarByID(avatarId);
            userDTO.setAvatarDTO(avatarDTO);
            userDTO = this.updateUserImp.update(userDTO);
            UserNoExtraDataDTO userNoExtraDataDTO = UserMapper.getInstance().dtoToDtoNoExtraData(userDTO);
            responseDTO.setData(userNoExtraDataDTO);
            responseDTO.addInfo("USER_ASIGNADO_AVATAR_OK");
        }catch(TfgException e){
        responseDTO.setErrorMessages(e.getErrorMessages());
    }catch (Exception e){
        responseDTO.addError(e.getMessage());
    }
        return responseDTO;
    }

}
