package com.example.authapi.users.infrastructure.entity;

import jakarta.persistence.*;
import jakarta.validation.constraints.NotNull;
import lombok.*;
import org.hibernate.annotations.NotFound;
import org.hibernate.annotations.NotFoundAction;

import java.util.HashSet;
import java.util.Objects;
import java.util.Set;

@Entity
@Data
@Getter
@AllArgsConstructor
@NoArgsConstructor
@Builder
public class UserEntity {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Integer id;

    @NotNull
    @Column(unique = true)
    private String username;

    @NotNull
    @Column(unique = true)
    private String email;

    private String password;
    private Integer id_UserSettings;
    @ManyToOne
    @JoinColumn(name = "avatar_id")
    @NotFound(action= NotFoundAction.IGNORE)
    private AvatarEntity avatarEntity;
    @NotNull
    @ManyToMany(fetch = FetchType.EAGER)
    @JoinTable(name = "userentity_rolentity", joinColumns = @JoinColumn(name = "userintenty_id"),
            inverseJoinColumns = @JoinColumn(name = "rolentity_id"))
    private Set<RolEntity> roles = new HashSet<>();

    @Column(columnDefinition="Boolean default '0'")
    private Boolean isEmailVerify;

    @Override
    public String toString() {
        return "UserEntity{" +
                "id=" + id +
                ", username='" + username + '\'' +
                ", email='" + email + '\'' +
                ", password='" + password + '\'' +
                ", id_UserSettings=" + id_UserSettings +
                ", avatarEntity_ID=" + avatarEntity.getId() +
                ", roles=" + roles +
                ", isEmailVerify=" + isEmailVerify +
                '}';
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        UserEntity entity = (UserEntity) o;
        return Objects.equals(id, entity.id) && Objects.equals(username, entity.username) && Objects.equals(email, entity.email) && Objects.equals(password, entity.password) && Objects.equals(id_UserSettings, entity.id_UserSettings) && Objects.equals(avatarEntity, entity.avatarEntity) && Objects.equals(roles, entity.roles) && Objects.equals(isEmailVerify, entity.isEmailVerify);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, username, email, password, id_UserSettings, avatarEntity, roles, isEmailVerify);
    }
}
