package com.example.authapi.users.infrastructure.mapper;

import com.example.authapi.users.application.dto.AvatarDTO;
import com.example.authapi.users.infrastructure.entity.AvatarEntity;
import com.example.authapi.users.infrastructure.entity.UserEntity;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class AvatarMapper implements Serializable {
    private static AvatarMapper instance;

    private AvatarMapper(){}

    public static AvatarMapper getInstance(){
        if(instance==null){
            instance = new AvatarMapper();
        }
        return instance;
    }

    public AvatarDTO entityToDto(AvatarEntity avatarEntity){
        if(avatarEntity == null){
            return null;
        }
        List<Integer> integerList;
        if(avatarEntity.getUserEntity()==null){
            integerList=null;
        }else{
            integerList = avatarEntity.getUserEntity().stream().map(UserEntity::getId).distinct().collect(Collectors.toList());
        }
        return AvatarDTO.builder()
                .id(avatarEntity.getId())
                .userId(integerList)
                .path(avatarEntity.getPath())
                .url(avatarEntity.getUrl())
                .type(avatarEntity.getType())
                .build();
    }

    public AvatarEntity dtoToEntity(AvatarDTO avatarDTO){
        if(avatarDTO ==null){
            return null;
        }
        return AvatarEntity.builder()
                .id(avatarDTO.getId())
                .userEntity(UserMapper.getInstance().listDtoToEntityList(avatarDTO.getUserDTO()))
                .path(avatarDTO.getPath())
                .url(avatarDTO.getUrl())
                .type(avatarDTO.getType())
                .build();
    }

    public List<AvatarEntity> listDtoToEntityList(List<AvatarDTO> avatarDTOList){
        if(avatarDTOList==null){
            return Collections.emptyList();
        }
        return avatarDTOList.stream().map(this::dtoToEntity).collect(Collectors.toList());
    }

    public List<AvatarDTO> listEntityToDtoList(List<AvatarEntity> avatarEntityList){
        if(avatarEntityList==null){
            return Collections.emptyList();
        }
        return avatarEntityList.stream().map(this::entityToDto).collect(Collectors.toList());
    }


}
