package com.example.authapi.users.infrastructure.mapper;

import com.example.authapi.users.application.dto.RolDTO;
import com.example.authapi.users.application.dto.UserDTO;
import com.example.authapi.users.application.dto.UserNoExtraDataDTO;
import com.example.authapi.users.infrastructure.entity.AvatarEntity;
import com.example.authapi.users.infrastructure.entity.RolEntity;
import com.example.authapi.users.infrastructure.entity.UserEntity;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
@Service
public class UserMapper implements Serializable {
    private static UserMapper instance;

    private UserMapper(){}

    public static UserMapper getInstance(){
        if(instance==null){
            instance = new UserMapper();
        }
        return instance;
    }

    public UserNoExtraDataDTO dtoToDtoNoExtraData(UserDTO userDTO) {
        if(userDTO == null){
            return null;
        }
        return UserNoExtraDataDTO.builder()
                .id(userDTO.getId())
                .email(userDTO.getEmail())
                .username(userDTO.getUsername())
                .urlAvatar(userDTO.getUrl())
                .build();
    }

    public UserDTO entityToDto(UserEntity user){
        if(user == null){
            return null;
        }
        Set<RolDTO> rolDTOSet = RolMapper.getInstance().setEntityToDtoSet(user.getRoles());
        return UserDTO.builder()
                .id(user.getId())
                .email(user.getEmail())
                .username(user.getUsername())
                .password(user.getPassword())
                .id_UserSettings(user.getId_UserSettings())
                .url(user.getAvatarEntity()==null?null:user.getAvatarEntity().getUrl())
                .roles(rolDTOSet)
                .isEmailVerify(user.getIsEmailVerify())
                .build();
    }

    public UserEntity dtoToEntity(UserDTO userDTO){
        if(userDTO ==null){
            return null;
        }
        AvatarEntity avatarEntity = AvatarMapper.getInstance().dtoToEntity(userDTO.getAvatarDTO());
        Set<RolEntity> rolEntityList = RolMapper.getInstance().setDtoToEntitySet(userDTO.getRoles());
        return UserEntity.builder()
                .id(userDTO.getId() != null ? userDTO.getId() : null)
                .email(userDTO.getEmail() != null ? userDTO.getEmail() : null)
                .password(userDTO.getPassword())
                .username(userDTO.getUsername() != null ? userDTO.getUsername() : null)
                .id_UserSettings(userDTO.getId_UserSettings())
                .avatarEntity(avatarEntity)
                .roles(rolEntityList)
                .isEmailVerify(userDTO.getIsEmailVerify() != null ? userDTO.getIsEmailVerify():false)
                .build();
    }

    public List<UserEntity> listDtoToEntityList(List<UserDTO> userDTOList){
        if(userDTOList==null){
            return Collections.emptyList();
        }
        return userDTOList.stream().map(this::dtoToEntity).collect(Collectors.toList());
    }

    public List<UserDTO> listEntityToDtoList(List<UserEntity> userEntityList){
        if(userEntityList==null){
            return Collections.emptyList();
        }
        return userEntityList.stream().map(this::entityToDto).collect(Collectors.toList());
    }

    public List<UserNoExtraDataDTO> listEntityToNoExtraDataDtoList(List<UserDTO> userDTOList){
        if(userDTOList==null){
            return Collections.emptyList();
        }
        return userDTOList.stream().map(this::dtoToDtoNoExtraData).collect(Collectors.toList());
    }

    public UserDTO objectAuthToDto(Object principal) {
        principal.getClass();
        UserDTO userDTO;
        userDTO = (UserDTO) principal;
        return  userDTO;
    }

/*
    public static User buildUserGrantedAuthority(User usuario){
        List<GrantedAuthority> authorities =
                usuario.getRoles().stream().map(rol -> new SimpleGrantedAuthority(rol
                        .getName())).collect(Collectors.toList());
        return User.builder()
                .id(usuario.getId())
                .email(usuario.getEmail())
                .username(usuario.getUsername())
                .roles(usuario.getRoles())
                .authorities(usuario.getAuthorities())
                .build();
    }

 */
}
