package com.example.authapi.users.infrastructure.mapper;

import com.example.authapi.shared.enums.LanguageEnum;
import com.example.authapi.shared.enums.VisualizationTypeEnum;
import com.example.authapi.users.application.dto.UserSettingsDTO;
import com.example.authapi.users.infrastructure.entity.UserSettingsEntity;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class UserSettingsMapper implements Serializable {
    private static UserSettingsMapper instance;

    private UserSettingsMapper(){}

    public static UserSettingsMapper getInstance(){
        if(instance==null){
            instance = new UserSettingsMapper();
        }
        return instance;
    }

    public UserSettingsDTO entityToDto(UserSettingsEntity userSettingsEntity){
        if(userSettingsEntity == null){
            return null;
        }
        return UserSettingsDTO.builder()
                .id(userSettingsEntity.getId())
                .languageEnum(LanguageEnum.valueOf(userSettingsEntity.getLanguageEnum()))
                .visualizationTypeEnum(VisualizationTypeEnum.valueOf(userSettingsEntity.getVisualizationTypeEnum()))
                .build();
    }

    public UserSettingsEntity dtoToEntity(UserSettingsDTO userSettingsDTO){
        if(userSettingsDTO ==null){
            return null;
        }
        return UserSettingsEntity.builder()
                .id(userSettingsDTO.getId())
                .languageEnum(userSettingsDTO.getLanguageEnum().name())
                .visualizationTypeEnum(userSettingsDTO.getVisualizationTypeEnum().name())
                .build();
    }

    public List<UserSettingsEntity> listDtoToEntityList(List<UserSettingsDTO> userSettingsDTOList){
        if(userSettingsDTOList==null){
            return Collections.emptyList();
        }
        return userSettingsDTOList.stream().map(this::dtoToEntity).collect(Collectors.toList());
    }

    public List<UserSettingsDTO> listEntityToDtoList(List<UserSettingsEntity> userSettingsEntityList){
        if(userSettingsEntityList==null){
            return Collections.emptyList();
        }
        return userSettingsEntityList.stream().map(this::entityToDto).collect(Collectors.toList());
    }

}
