package com.example.authapi.users.infrastructure.repository_outputAdapter;

import com.example.authapi.users.application.dto.AvatarDTO;
import com.example.authapi.users.application.repository.AvatarRepository;
import com.example.authapi.users.infrastructure.entity.AvatarEntity;
import com.example.authapi.users.infrastructure.mapper.AvatarMapper;
import com.example.authapi.users.infrastructure.repository_outputPort.SqlAvatarRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class SqlAvatarRepositoryImp implements AvatarRepository {

    private final SqlAvatarRepository sqlAvatarRepository;

    @Autowired
    public SqlAvatarRepositoryImp(SqlAvatarRepository sqlAvatarRepository) {
        this.sqlAvatarRepository = sqlAvatarRepository;
    }

    @Override
    public AvatarDTO create(AvatarDTO avatarDTO) {
        return AvatarMapper.getInstance().entityToDto(sqlAvatarRepository.save(AvatarMapper.getInstance().dtoToEntity(avatarDTO)));

    }

    @Override
    public AvatarDTO getById(Integer id) {
        return AvatarMapper.getInstance().entityToDto(this.sqlAvatarRepository.findById(id).orElse(null));
    }

    @Override
    public AvatarDTO findByPath(String path) {
        return AvatarMapper.getInstance().entityToDto(this.sqlAvatarRepository.findByPath(path).orElse(null));
    }

    @Override
    public AvatarDTO findByUrl(String url) {
        return AvatarMapper.getInstance().entityToDto(this.sqlAvatarRepository.findByUrl(url).orElse(null));
    }

    @Override
    public AvatarDTO update(AvatarDTO avatarDTO) {
        System.out.println("Update Avatar Not Allowed");
        return null;
    }

    @Override
    public void delete(AvatarDTO avatarDTO) {
        this.sqlAvatarRepository.delete(AvatarMapper.getInstance().dtoToEntity(avatarDTO));
    }

    @Override
    public List<AvatarDTO> getAll() {
        List<AvatarEntity> avatarEntityList = this.sqlAvatarRepository.findAll();
        return AvatarMapper.getInstance().listEntityToDtoList(avatarEntityList);
    }
}
