package com.example.authapi.users.infrastructure.repository_outputAdapter;

import com.example.authapi.users.application.dto.UserDTO;
import com.example.authapi.users.application.repository.UserRepository;
import com.example.authapi.users.infrastructure.entity.UserEntity;
import com.example.authapi.users.infrastructure.mapper.UserMapper;
import com.example.authapi.users.infrastructure.repository_outputPort.SqlUserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

@Service
public class SqlUserRepositoryImp implements UserRepository {

    private final SqlUserRepository sqlUserRepository;
    PasswordEncoder passwordEncoder;

    @Autowired
    public SqlUserRepositoryImp(SqlUserRepository sqlUserRepository,PasswordEncoder passwordEncoder) {
        this.sqlUserRepository = sqlUserRepository;
        this.passwordEncoder=passwordEncoder;
    }

    @Override
    public UserDTO create(UserDTO userDTO) {
        UserEntity userEntity = UserMapper.getInstance().dtoToEntity(userDTO);
        this.encode(userEntity);
        return UserMapper.getInstance().entityToDto(this.sqlUserRepository.save(Objects.requireNonNull(userEntity)));
    }

    private void encode(UserEntity userEntity) {
        String passwd = userEntity.getPassword();
        if (passwd!=null && passwd.length()!=60){
            passwd=passwordEncoder.encode(passwd);
        }
        userEntity.setPassword(passwd);
    }

    @Override
    public UserDTO delete(UserDTO userDTO) {
        UserEntity userEntity = UserMapper.getInstance().dtoToEntity(userDTO);
        this.sqlUserRepository.delete(Objects.requireNonNull(userEntity));
        return userDTO;
    }

    @Override
    public UserDTO getById(Integer userId) {
        return UserMapper.getInstance().entityToDto(this.sqlUserRepository.findById(userId).orElse(null));
    }

    @Override
    public UserDTO findByEmail(String email) {
        return UserMapper.getInstance().entityToDto(this.sqlUserRepository.findByEmail(email));
    }

    public UserDTO findByUsername(String username) {
        return UserMapper.getInstance().entityToDto(this.sqlUserRepository.findByUsername(username));
    }

    @Override
    public UserDTO update(UserDTO userDTO) {
        UserEntity userEntity = UserMapper.getInstance().dtoToEntity(userDTO);
        this.encode(userEntity);
        UserEntity user=sqlUserRepository.save(Objects.requireNonNull(userEntity));
        return UserMapper.getInstance().entityToDto(user);

    }

    @Override
    public List<UserDTO> getAll() {
        return UserMapper.getInstance().listEntityToDtoList(this.sqlUserRepository.findAll());
    }

    @Override
    public UserDTO findByUsernameOrEmail(String username, String email) {
        UserDTO userDTO2 = this.findByEmail(email);
        UserDTO userDTO1 = this.findByUsername(username);
        return userDTO1 ==null? userDTO2 : userDTO1;
    }

}
