package com.example.braintreeapi.service;

import com.braintreegateway.*;
import com.example.braintreeapi.config.BraintreeConfig;
import com.example.braintreeapi.dto.ClientTokenDto;
import com.example.braintreeapi.dto.PurchaseDto;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class PaymentService {

    @Autowired
    BraintreeConfig braintreeConfig;

    public BraintreeGateway getGateway() {
        log.info("...getGateway...");
        return new BraintreeGateway(
                Environment.SANDBOX,
                braintreeConfig.getMerchantId(),
                braintreeConfig.getPublicKey(),
                braintreeConfig.getPrivateKey()
        );
    }

    public ClientTokenDto getToken(){
        log.info("getToken...");
        ClientTokenRequest request = new ClientTokenRequest();
        String token = getGateway().clientToken().generate(request);
        log.info("...getToken");
        return new ClientTokenDto(token);
    }

    public Result<Transaction> checkout(PurchaseDto dto){
        log.info("checkout...");
        TransactionRequest request = new TransactionRequest()
                .amount(dto.getAmount())
                .paymentMethodNonce(dto.getNonce())
                .options()
                .submitForSettlement(true)
                .done();
        log.info("...checkout");
        return getGateway().transaction().sale(request);
    }


}
