import {Component} from "react";
import * as Yup from "yup";
import * as singleSpa from 'single-spa';
import {ErrorMessage, Field, Form, Formik} from "formik";
import './stylesLoginRegister.css';
import {Link} from "react-router-dom";
import {state$} from "@TfgApp/mf-utility";

type Props = {};
type State = {
    redirect: string | null,
    username: string,
    password: string,
    loading: boolean,
    message: string
};

export default class Login extends Component<Props, State> {
    constructor(props: Props) {
        super(props);
        this.handleLogin = this.handleLogin.bind(this);

        this.state = {
            redirect: null,
            username: "",
            password: "",
            loading: false,
            message: ""
        };
    }

    extracted2() {
        const subscription = state$.subscribe((data) => {
        });
        state$.next({isLogged: true})
        return () => {
            subscription.unsubscribe();
        }
    }
    
    validationSchema() {
        return Yup.object().shape({
            username: Yup.string().required("This field is required!"),
            password: Yup.string().required("This field is required!"),
        });
    }


    handleLogin(formValue: { username: string; password: string }) {
        const { username, password } = formValue;

        this.setState({
            message: "",
            loading: true
        });
        
        const options = {
            method: 'POST',
            headers: {
            'Content-Type': 'application/json',
            },
            body: JSON.stringify({username,password}),
            };
        fetch("http://localhost:5000/auth/login",options)
            .then(data => {
                if (!data.ok) {
                    console.error("codigo de error: "+data.status+" mensaje: credenciales no validas"," texto: ",data.statusText)
                    throw Error(data.status.toString()+": "+data.statusText+ " credenciales no validas");
                }
                return data.json();
                }).then(update => {
                    console.log(update);
                    localStorage.setItem("auth",update.token);
                    //extracted2();
                    this.extracted2();
                    singleSpa.navigateToUrl("/resources/communityCourses");
                    return;
                }).catch(e => {
                    console.error(e);
                    this.setState({
                        message: "credenciales no validas, o email no verificado",
                        loading: false
                    });
                    
                });

/*
        AuthService.login(username, password).then(
            () => {
                this.setState({
                    redirect: "/profile"
                });
            },
            error => {
                const resMessage =
                    (error.response &&
                        error.response.data &&
                        error.response.data.message) ||
                    error.message ||
                    error.toString();

                this.setState({
                    loading: false,
                    message: resMessage
                });
            }
        );
        */
    }
    render() {

        const { loading, message } = this.state;

        const initialValues = {
            username: "",
            password: "",
        };

        return (
            <div className="col-sm-12 col-md-8 m-auto" >
                <div className="card card-container">
                    <h1 className="text-center">Login</h1>
                    <Formik
                        initialValues={initialValues}
                        validationSchema={this.validationSchema}
                        onSubmit={this.handleLogin}
                    >
                        <Form>
                            <div className="form-group">
                                <label htmlFor="username">Username</label>
                                <Field name="username" type="text" className="form-control" placeholder="usuario o email"/>
                                <ErrorMessage
                                    name="username"
                                    component="div"
                                    className="alert alert-danger"
                                />
                            </div>

                            <div className="form-group">
                                <label htmlFor="password">Password</label>
                                <Field name="password" type="password" className="form-control" />
                                <ErrorMessage
                                    name="password"
                                    component="div"
                                    className="alert alert-danger"
                                />
                            </div>

                            <div className="form-group m-3 text-center">
                                <button type="submit" className="btn btn-primary btn-block" disabled={loading}>
                                    {loading && (
                                        <span className="spinner-border spinner-border-sm"></span>
                                    )}
                                    <span>Login</span>
                                </button>
                            </div>

                            <div className="form-group">
                                <Link id={"link"} to="/home/register">
                                    <label>¿Todavía no tienes cuenta?</label>
                                </Link>
                            </div>

                            {message && (
                                <div className="form-group">
                                    <div className="alert alert-danger" role="alert">
                                        {message}
                                    </div>
                                </div>
                            )}
                        </Form>
                    </Formik>
                </div>
            </div>
        );
    }
}
