import {Component} from "react";
//import { Navigate } from "react-router-dom";
import { Formik, Field, Form, ErrorMessage } from "formik";
import * as Yup from "yup";
import * as singleSpa from 'single-spa';
import './stylesLoginRegister.css';
import {Link} from "react-router-dom";

type Props = {};
type State = {
    username: string,
    email: string,
    password: string,
    successful: boolean,
    message: string
};

export default class Register extends Component<Props, State> {
    constructor(props: Props) {
        super(props);
        this.handleRegister = this.handleRegister.bind(this);

        this.state = {
            username: "",
            email: "",
            password: "",
            successful: false,
            message: ""
        };
    }

    validationSchema() {
        return Yup.object().shape({
            username: Yup.string()
                .test(
                    "len",
                    "The username must be between 3 and 20 characters.",
                    (val: any) =>
                        val &&
                        val.toString().length >= 3 &&
                        val.toString().length <= 20
                )
                .required("This field is required!"),
            email: Yup.string()
                .email("This is not a valid email.")
                .required("This field is required!"),
            password: Yup.string()
                .test(
                    "len",
                    "The password must be between 6 and 40 characters.",
                    (val: any) =>
                        val &&
                        val.toString().length >= 6 &&
                        val.toString().length <= 40
                )
                .required("This field is required!"),
        });
    }

    handleRegister(formValue: { username: string; email: string; password: string; repeatPassword }) {
        const { username, email, password, repeatPassword } = formValue;

        this.setState({
            message: "",
            successful: false
        });

        const options = {
            method: 'POST',
            headers: {
            'Content-Type': 'application/json',
            },
            body: JSON.stringify({username,email,password,repeatPassword}),
            };
        fetch("http://localhost:5000/auth/create",options)
            .then(data => {
                if (!data.ok) {
                    console.error("codigo de error: "+data.status+" mensaje: ",data.statusText)
                    throw Error(data.status.toString()+": "+data.statusText);
                }
                return data.json();
                }).then(update => {
                    console.log(update);
                    //localStorage.setItem("auth",update.token)
                    singleSpa.navigateToUrl("/home/aviso")
                }).catch(e => {
                    console.error(e);
                    this.setState({message: "username ya existe o email ya existe",
                                    successful: false})
                });

        /*
        AuthService.register(
            username,
            email,
            password
        ).then(
            response => {
                this.setState({
                    message: response.data.message,
                    successful: true
                });
            },
            error => {
                const resMessage =
                    (error.response &&
                        error.response.data &&
                        error.response.data.message) ||
                    error.message ||
                    error.toString();

                this.setState({
                    successful: false,
                    message: resMessage
                });
            }
        );
        */
    }


    validatePassword = (values) => {
        //entra al clicar en el submit button
        console.log("entro 1 validatePassword")
        console.log("values", values)
        console.log("salgo 1 validatePassword")
        let error = "";
        const passwordRegex = /(?=.*[0-9])/;
        if (!values) {
            error = "*Required";
        } else if (values.length < 8) {
            error = "*Password must be 8 characters long.";
        } else if (!passwordRegex.test(values)) {
            error = "*Invalid password. Must contain one number.";
        }else this.setState({password: values});
        return error;
    };

    validateConfirmPassword = (pass, value) => {
        //entra siempre
        console.log("entro 2 validateConfirmPassword")
        console.log("pass", pass)
        console.log("value", value)
        console.log("salgo 2 validateConfirmPassword")
        let error = "";
        if (pass && value) {
            if (pass !== value) {
                error = "Password not matched";
            }
        }
        return error;
    };

    render() {
        const { successful, message } = this.state;

        const initialValues = {
            username: "",
            email: "",
            password: "",
            repeatPassword: "",
        };

        return (
            <div className="col-sm-12 col-md-8 m-auto">
                <div className="card card-container">
                    <h1 className="text-center">Register</h1>
                    <Formik
                        initialValues={initialValues}
                        validationSchema={this.validationSchema}
                        onSubmit={this.handleRegister}
                    >
                        <Form>
                            {!successful && (
                                <div>
                                    <div className="form-group">
                                        <label htmlFor="username"> Username </label>
                                        <Field name="username" type="text" className="form-control" />
                                        <ErrorMessage
                                            name="username"
                                            component="div"
                                            className="alert alert-danger"
                                        />
                                    </div>

                                    <div className="form-group">
                                        <label htmlFor="email"> Email </label>
                                        <Field name="email" type="email" className="form-control" />
                                        <ErrorMessage
                                            name="email"
                                            component="div"
                                            className="alert alert-danger"
                                        />
                                    </div>

                                    <div className="form-group">
                                        <label htmlFor="password"> Password </label>
                                        <Field
                                            name="password"
                                            type="password"
                                            className="form-control"
                                            validate={value => this.validatePassword(value)}
                                        />
                                        <ErrorMessage
                                            name="password"
                                            component="div"
                                            className="alert alert-danger"
                                        />
                                    </div>

                                    <div className="form-group">
                                        <label htmlFor="repeatPassword"> Repeat Password </label>
                                        <Field
                                            name="repeatPassword"
                                            type="password"
                                            className="form-control"
                                            validate={value =>
                                                this.validateConfirmPassword(this.state.password, value)
                                            }
                                        />
                                        <ErrorMessage
                                            name="repeatPassword"
                                            component="div"
                                            className="alert alert-danger"
                                        />
                                    </div>

                                    <div className="form-group m-3 text-center">
                                        <button type="submit" className="btn btn-primary btn-block">Sign Up</button>
                                    </div>

                                    <div className="form-group">
                                        <Link id="link" to="/home/login">
                                            <label>¿Ya tienes una cuenta?</label>
                                        </Link>
                                    </div>

                                </div>
                            )}

                            {message && (
                                <div className="form-group">
                                    <div
                                        className={
                                            successful ? "alert alert-success" : "alert alert-danger"
                                        }
                                        role="alert"
                                    >
                                        {message}
                                    </div>
                                </div>
                            )}
                        </Form>
                    </Formik>
                </div>
            </div>
        );
    }
}
