import {Component, OnInit} from '@angular/core';
import {map, Observable, Subscription} from "rxjs";
import {PaymentService} from "../service/payment.service";
import {PurchaseDto} from "../dto/purchase.dto";
import {Router} from "@angular/router";
import {state$,state2$} from '@TfgApp/mf-utility';
import * as singleSpa from "single-spa";

@Component({
  selector: 'app-payment',
  templateUrl: './payment.component.html',
  styleUrls: ['./payment.component.css']
})
export class PaymentComponent implements OnInit{
  title = 'TfgAppBraintree';
  amount: number = 0;
  subscription!: Subscription;
  subscription2!: Subscription;
  isdataItems: any;
  dataStorageItems: any = [];
  listDatos: any = [];
  CurrentUser: any = {}

  constructor(private paymentService: PaymentService,
              private router: Router) {
  }

  ngOnInit(): void {
    this.getUserByToken()
    this.subscription2 = state2$.subscribe( (data: any) => {
      console.log("payment state", data.nItems)
      if(data.nItems || (localStorage.getItem("items")!=null && localStorage.getItem("items")!=undefined)){
        if(data.nItems === undefined){
          state2$.next({nItems: true})
        }
        this.isdataItems=true
        this.dataStorageItems=localStorage.getItem("items")
        this.dataStorageItems=JSON.parse(this.dataStorageItems)
        this.calcularTotalAmount();
      }
    })
    console.log('this.dataItems',this.dataStorageItems)
    this.subscription2.unsubscribe();
  }

  onPaymentStatus(event: any): void {
  }

  getClientToken(): Observable<any>{
    return this.paymentService.getToken().pipe(map(
      data => {
        return data.token;
      }/*,
        err => {
        return Observable.throw(err);
      }*/
    ));
  }

  checkout(nonce: string, amount: number): Observable<any> {
    //console.log('nonce: ' + nonce);
    //console.log('amount: ' + amount);
    const dto = new PurchaseDto(nonce,amount);
    return this.paymentService.checkout(dto).pipe(map(
      data => {
        //console.log("entro")
        this.llamarParaAsignarCursosEnBackend()
        this.clearLocalStorage()
        this.subscription2 = state2$.subscribe( (data: any) => {
        })
        state2$.next({nItems: false})
        this.subscription2.unsubscribe();
        this.router.navigate(["/braintree/success"]);
        return data;
      }/*,
        err => {
        return Observable.throw(err);
      }*/
    ));
  }

  async getUserByToken() {
    const token = localStorage.getItem("auth")
    let user
    let datos
    if (token!==undefined && token!==null){
      try{
        user = await fetch("http://127.0.0.1:5000/auth/tokentouser/"+token)
        datos= await user.json()
        console.log("user: ",user)
        console.log("user.json: ",datos)
        this.CurrentUser=datos
        console.log("this.CurrentUser",this.CurrentUser)
      }catch (error) {
        localStorage.removeItem("auth")
        this.subscription = state$.subscribe( (data: any) => {})
        state$.next({isLogged: false})
        this.subscription.unsubscribe();
        singleSpa.navigateToUrl("/home/login")
      }
    }
  }

  deleteItem(item: any, i: number) {
    this.dataStorageItems.splice(i,1);
    localStorage.setItem("items",JSON.stringify(this.dataStorageItems))
    if(this.dataStorageItems.length==0){
      localStorage.removeItem("items")
    }
    this.calcularTotalAmount();
  }

  async calcularTotalAmount(){
    let count = 0;
    for (const item of this.dataStorageItems) {
      //TODO: llamar al backend para coger el curso a traves del nombre y de allí saber el precio
      const resu=await fetch("http://127.0.0.1:5000/course/find/"+item.name)
      const datos=await resu.json()
      this.listDatos.push(datos.data);
      count = count + datos.data.price;
    }
    this.amount=count;
    console.log('this.amount',this.amount)
  }

  private async llamarParaAsignarCursosEnBackend() {
    //TODO llamar al Backend para asignarle el rol de estudiante a esos cursos
    //console.log("this.listDatos",this.listDatos)
    try {
      for (const item of this.listDatos) {
        const resu=await fetch("http://127.0.0.1:5000/course/estudiante/"+item.id+"/"+this.CurrentUser.id)
        const datos=await resu.json()
      }
    }catch (error) {
      console.error(error)
    }
    console.log('aqui se deberán asignar los cursos al usuario actual')
  }

  private clearLocalStorage() {
    this.amount=0;
    this.isdataItems=false;
    this.dataStorageItems=[]
    localStorage.removeItem("items")
  }

  productsFree() {
    this.clearLocalStorage();
    singleSpa.navigateToUrl("/resources/myCourses") //-> INFO: viajar por microfrontends
  }

  goCommunityCourses() {
    this.clearLocalStorage();
    singleSpa.navigateToUrl("/resources/communityCourses") //-> INFO: viajar por microfrontends
  }
}
