import {Component, OnInit} from '@angular/core';
import * as singleSpa from 'single-spa';
import {state$,state2$} from '@TfgApp/mf-utility';
import {Subscription} from "rxjs";
import {assetUrl} from "../single-spa/asset-url";
@Component({
  selector: 'app-root-navbar',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.css']
})
export class AppComponent implements OnInit{
  title = 'TfgAppNavbar';
  userLogged: any = false;
  dataIsLogged: any;
  subscription!: Subscription;
  subscription2!: Subscription;
  auth: string | null | undefined;
  items: any;
  areItems: any = false;
  sizeItems: number = 0;
  imageNav: any;
  avatarUser: any;

  ngOnInit(): void {
    let s =  "/favicon.ico";
    this.imageNav = assetUrl(s);

    this.subscription = state$.subscribe( (data: any) => {
      this.checkIfUserLogged();
      this.dataIsLogged = data.isLogged;
    })
    this.subscription2 = state2$.subscribe( (data: any) => {
      this.areItems = data.nItems;
      this.checkNumItems(data);
    })

  }

  ngOnDestroy(): void {
    this.subscription?.unsubscribe();
    this.subscription2?.unsubscribe();
  }

  async checkIfUserLogged(){
    this.auth=localStorage.getItem("auth")
    if (this.dataIsLogged || (this.auth!==undefined && this.auth!==null)){
      this.userLogged = true;
      await this.setCurrentUser();
    }
    else{
      this.userLogged = false;
    }
  }

  checkNumItems(data: any) {
    console.log("NAV checkNumItems")
    this.items=localStorage.getItem("items")
    this.items=JSON.parse(this.items)
    if (this.items!==undefined && this.items!==null){
      this.areItems = true;
      this.sizeItems = this.items.length;
    }else{
      this.areItems = false;
    }
  }

  private async setCurrentUser() {
    try {
      const idPerson=await fetch("http://127.0.0.1:5000/auth/tokentouser/"+this.auth)
      const idPersonJson=await idPerson.json()
      console.log("idPerson",idPersonJson)
      const id=await idPersonJson.id;
      this.avatarUser=idPersonJson.urlAvatar
    }catch (error) {
      localStorage.removeItem("auth")
      this.dataIsLogged=false
      this.userLogged = false;
      singleSpa.navigateToUrl("/home/login")
    }
  }

  goLogin() {
    singleSpa.navigateToUrl("/home/login") //-> INFO: viajar por microfrontends
  }
  goRegister() {
    singleSpa.navigateToUrl("/home/register") //-> INFO: viajar por  componentes
  }
  goPayment() {
    singleSpa.navigateToUrl("/braintree/payment") //-> INFO: viajar por  componentes
  }
  goCommunityCourses() {
    singleSpa.navigateToUrl("/resources/communityCourses") //-> INFO: viajar por microfrontends
  }
  goMyCourses() {
    singleSpa.navigateToUrl("/resources/myCourses") //-> INFO: viajar por microfrontends
  }
  goMyProfile() {
    singleSpa.navigateToUrl("/resources/myProfile") //-> INFO: viajar por microfrontends
  }
  goSettings() {
    singleSpa.navigateToUrl("/resources/mySettings") //-> INFO: viajar por microfrontends
  }
  goLogout() {
    localStorage.removeItem("auth")
    state$.next({isLogged: false})
    singleSpa.navigateToUrl("/") //-> INFO: viajar por microfrontends
  }
  createCourse() {
    singleSpa.navigateToUrl("/resources/createCourses") //-> INFO: viajar por microfrontends
  }



}
