import { enableProdMode, NgZone } from '@angular/core';

import { platformBrowserDynamic } from '@angular/platform-browser-dynamic';
import { Router, NavigationStart } from '@angular/router';

import { singleSpaAngular, getSingleSpaExtraProviders } from 'single-spa-angular';


import { AppModule } from './app/app.module';
import { environment } from './environments/environment';
import { singleSpaPropsSubject } from './single-spa/single-spa-props';

if (environment.production) {
  enableProdMode();
}

const lifecycles = singleSpaAngular({
  bootstrapFunction: singleSpaProps => {
    singleSpaPropsSubject.next(singleSpaProps);
    configFirebase();
    return platformBrowserDynamic(getSingleSpaExtraProviders()).bootstrapModule(AppModule);
  },
  template: '<app-root-resources />',
  Router,
  NavigationStart,
  NgZone,
});

export const bootstrap = lifecycles.bootstrap;
export const mount = lifecycles.mount;
export const unmount = lifecycles.unmount;


function configFirebase(){
  (async () => {
    environment.firebaseConfig = await fetch("http://127.0.0.1:5000/firebase/config")
      .then(async (res) => {
        const info = await res.json()
        console.log('info main', info)
        //@ts-ignore
        const infoData = info.data;
        return {
          apiKey: infoData["apiKey"],
          authDomain: infoData["authDomain"],
          projectId: infoData["projectId"],
          storageBucket: infoData["storageBucket"],
          messagingSenderId: infoData["messagingSenderId"],
          appId: infoData["appId"],
        }
      });
  })();
}
