package com.example.resapi.courses.application.course.update;


import com.example.resapi.courses.application.dto.CourseDTO;
import com.example.resapi.courses.application.mapper.CourseMapperApplication;
import com.example.resapi.courses.domain.entity.CourseDomain;
import com.example.resapi.courses.domain.exceptions.ExceptionXNotExistException;
import com.example.resapi.courses.application.repository.CourseRepository;
import com.example.resapi.courses.infrastructure.mapper.CourseMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class UpdateCourseImp implements UpdateCourse {

    private final CourseRepository courseRepository;

    @Autowired
    public UpdateCourseImp(CourseRepository courseRepository) {
        this.courseRepository = courseRepository;
    }

    @Override
    public CourseDTO getCourseById(Integer id){
        CourseDTO courseDTO = courseRepository.getById(id);
        if(courseDTO == null){
            throw new ExceptionXNotExistException("Course");
        }
        return courseDTO;
    }

    @Override
    public CourseDTO updateName(Integer id, String new_name) {
        CourseDomain course = CourseMapperApplication.getInstance().dtoToDomain(this.getCourseById(id));
        course.rename(new_name);
        CourseDTO courseDTO = CourseMapperApplication.getInstance().domainToDto(course);
        courseRepository.update(courseDTO);
        return courseDTO;
    }

    @Override
    public CourseDTO addProfessor(Integer id_course, Integer id_professor) {
        CourseDomain course = CourseMapperApplication.getInstance().dtoToDomain(this.getCourseById(id_course));
        course.addProfessor(id_professor);
        CourseDTO courseDTO = CourseMapperApplication.getInstance().domainToDto(course);
        courseRepository.update(courseDTO);
        return courseDTO;
    }

    @Override
    public CourseDTO addStudent(Integer id_course, Integer id_student) {
        CourseDomain course = CourseMapperApplication.getInstance().dtoToDomain(this.getCourseById(id_course));
        course.addStudent(id_student);
        CourseDTO courseDTO = CourseMapperApplication.getInstance().domainToDto(course);
        courseRepository.update(courseDTO);
        return courseDTO;
    }

    @Override
    public CourseDTO removeProfessor(Integer id_course, Integer id_professor) {
        CourseDomain course = CourseMapperApplication.getInstance().dtoToDomain(this.getCourseById(id_course));
        course.removeProfessor(id_professor);
        CourseDTO courseDTO = CourseMapperApplication.getInstance().domainToDto(course);
        courseRepository.update(courseDTO);
        return courseDTO;
    }

    @Override
    public CourseDTO removeStudent(Integer id_course, Integer id_student) {
        CourseDomain course = CourseMapperApplication.getInstance().dtoToDomain(this.getCourseById(id_course));
        course.removeProfessor(id_student);
        CourseDTO courseDTO = CourseMapperApplication.getInstance().domainToDto(course);
        courseRepository.update(courseDTO);
        return courseDTO;
    }

    @Override
    public CourseDTO update(CourseDTO course) {
        CourseDTO originalDto = courseRepository.getById(course.getId());
        return courseRepository.update(CourseMapper.getInstance().bothToDTO(course,originalDto));
    }

}
