package com.example.resapi.courses.application.mapper;

import com.example.resapi.courses.application.dto.ChatMessageDTO;
import com.example.resapi.courses.domain.entity.ChatMessageDomain;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class ChatMessageMapperApplication implements Serializable {
    private static ChatMessageMapperApplication instance;

    private ChatMessageMapperApplication(){}

    public static ChatMessageMapperApplication getInstance(){
        if(instance==null){
            instance = new ChatMessageMapperApplication();
        }
        return instance;
    }

    public ChatMessageDTO domainToDto(ChatMessageDomain chatMessageDomain){
        if(chatMessageDomain ==null){
            return null;
        }
        return ChatMessageDTO.builder()
                .id(chatMessageDomain.getId())
                .id_course(chatMessageDomain.getId_course())
                .id_account(chatMessageDomain.getId_account())
                .text(chatMessageDomain.getText())
                .time(chatMessageDomain.getTime())
                .build();
    }

    public ChatMessageDomain dtoToDomain(ChatMessageDTO chatMessageDTO){
        if(chatMessageDTO == null){
            return null;
        }
        return ChatMessageDomain.builder()
                .id(chatMessageDTO.getId())
                .id_course(chatMessageDTO.getId_course())
                .id_account(chatMessageDTO.getId_account())
                .text(chatMessageDTO.getText())
                .time(chatMessageDTO.getTime())
                .build();
    }

    public List<ChatMessageDTO> listDomainToListDto(List<ChatMessageDomain> chatMessageDomainList){
        if(chatMessageDomainList==null){
            return Collections.emptyList();
        }
        return chatMessageDomainList.stream().map(this::domainToDto).collect(Collectors.toList());
    }
    public List<ChatMessageDomain> listDtoToDomainList(List<ChatMessageDTO> chatMessageDTOList){
        if(chatMessageDTOList ==null){
            return Collections.emptyList();
        }
        return chatMessageDTOList.stream().map(this::dtoToDomain).collect(Collectors.toList());
    }


}
