package com.example.resapi.courses.application.mapper;

import com.example.resapi.courses.application.dto.CategoriesDTO;
import com.example.resapi.courses.application.dto.CourseDTO;
import com.example.resapi.courses.domain.entity.CategoriesDomain;
import com.example.resapi.courses.domain.entity.CourseDomain;
import org.json.JSONArray;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
@Service
public class CourseMapperApplication implements Serializable {
    private static CourseMapperApplication instance;

    private CourseMapperApplication(){}

    public static CourseMapperApplication getInstance(){
        if(instance==null){
            instance = new CourseMapperApplication();
        }
        return instance;
    }


    private Set<Integer> StrToSet (String str) {
        Set<Integer> set = new HashSet<>();
        JSONArray jsonArray = new JSONArray(str);
        for (Object i:jsonArray){
            set.add((Integer)i);
        }
        return set;
    }

    private String SetToStr (Set<Integer> set) {
        JSONArray jsonArray = new JSONArray(set);
        //String resu = "["+CDL.rowToString(jsonArray).substring(0,CDL.rowToString(jsonArray).length()-1)+"]";
        return jsonArray.toString();
        //String resu3 = jsonArray.toString(0);
    }

    public CourseDTO domainToDto(CourseDomain course) {
        if(course == null){
            return null;
        }
        List<CategoriesDomain> categoriesDomainList = course.getList_Categories();
        List<CategoriesDTO> categoriesDTOList = CategoriesMapperApplication.getInstance().listDomainToListDto(categoriesDomainList);

        CourseDTO dto= CourseDTO.builder()
                .id(course.getId())
                .name(course.getName())
                .price(course.getPrice())
                .list_Categories(categoriesDTOList)
                .list_id_professors(course.getList_id_professors())
                .list_id_students(course.getList_id_students())
                //.list_id_publications(course.getList_id_publications()) //TODO
                .build();
        return dto;
    }

    public CourseDomain dtoToDomain(CourseDTO courseDTO){
        if(courseDTO == null){
            return null;
        }
        List<CategoriesDomain> categoriesDomainList = CategoriesMapperApplication.getInstance().listDtoToDomainList(courseDTO.getList_Categories());

        return CourseDomain.builder()
                .id(courseDTO.getId())
                .name(courseDTO.getName())
                .price(courseDTO.getPrice())
                .list_Categories(categoriesDomainList)
                .list_id_professors(courseDTO.getList_id_professors())
                .list_id_students(courseDTO.getList_id_students())
                //.list_id_publications(getCourseDTO.getList_id_publications()) //TODO
                .build();
    }

    public CourseDTO objectToDto(Object obj) {
        if(obj == null){
            return null;
        }
        CourseDomain courseDomain = (CourseDomain) obj;
        return this.domainToDto(courseDomain);
    }


    public CourseDomain objectToDomain(Object obj) {
        if(obj == null){
            return null;
        }
        CourseDTO courseDTO = (CourseDTO) obj;
        return this.dtoToDomain(courseDTO);
    }

    public CourseDomain toDomain(CourseDTO courseDTO, CourseDomain course){
        if(courseDTO == null){
            return null;
        }
        if (courseDTO.getName()!=null && courseDTO.getName()!=""){
            course.setName(courseDTO.getName());
        }
        if (courseDTO.getPrice()!=null){
            course.setPrice(courseDTO.getPrice());
        }
        if (courseDTO.getList_Categories()!=null && !courseDTO.getList_Categories().isEmpty()){
            List<CategoriesDomain> categoriesDomainList = CategoriesMapperApplication.getInstance().listDtoToDomainList(courseDTO.getList_Categories());
            course.setList_Categories(categoriesDomainList);
        }
        if (courseDTO.getList_id_professors()!=null && !courseDTO.getList_id_professors().isEmpty()){
            course.setList_id_professors(courseDTO.getList_id_professors());
        }
        if (courseDTO.getList_id_students()!=null && !courseDTO.getList_id_students().isEmpty()) {
            course.setList_id_students(courseDTO.getList_id_students());
        }
        return course;
    }
    public List<CourseDTO> listDomainToListDto(List<CourseDomain> courseEntityList){
        if(courseEntityList==null){
            return Collections.emptyList();
        }
        return courseEntityList.stream().map(this::domainToDto).collect(Collectors.toList());
    }
    public List<CourseDomain> listDtoToDomainList(List<CourseDTO> courseEntityList){
        if(courseEntityList==null){
            return Collections.emptyList();
        }
        return courseEntityList.stream().map(this::dtoToDomain).collect(Collectors.toList());
    }



}
