package com.example.resapi.courses.application.mapper;

import com.example.resapi.courses.application.dto.FileDTO;
import com.example.resapi.courses.application.dto.PublicationDTO;
import com.example.resapi.courses.domain.entity.FileDomain;
import com.example.resapi.courses.domain.entity.PublicationDomain;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class FileMapperApplication implements Serializable {
    private static FileMapperApplication instance;

    private FileMapperApplication(){}

    public static FileMapperApplication getInstance(){
        if(instance==null){
            instance = new FileMapperApplication();
        }
        return instance;
    }


    public FileDTO domainToDto(FileDomain fileDomain) {
        if(fileDomain == null){
            return null;
        }
        PublicationDTO publicationDTO = PublicationMapperApplication.getInstance().domainToDto(fileDomain.getPublicationDomain());
        return FileDTO.builder()
                .id(fileDomain.getId())
                .id_account(fileDomain.getId_account())
                .publicationDTO(publicationDTO)
                .path(fileDomain.getPath())
                .url(fileDomain.getUrl())
                .type(fileDomain.getType())
                .build();
    }

    public FileDomain dtoToDomain(FileDTO fileDTO) {
        if(fileDTO == null){
            return null;
        }
        PublicationDomain publicationDomain = PublicationMapperApplication.getInstance().dtoToDomain(fileDTO.getPublicationDTO());

        return FileDomain.builder()
                .id(fileDTO.getId())
                .id_account(fileDTO.getId_account())
                .publicationDomain(publicationDomain)
                .path(fileDTO.getPath())
                .url(fileDTO.getUrl())
                .type(fileDTO.getType())
                .build();
    }

    public List<FileDTO> listDomainToListDto(List<FileDomain> fileDomainList){
        if(fileDomainList==null){
            return Collections.emptyList();
        }
        return fileDomainList.stream().map(this::domainToDto).collect(Collectors.toList());
    }
    public List<FileDomain> listDtoToDomainList(List<FileDTO> fileDTOList){
        if(fileDTOList ==null){
            return Collections.emptyList();
        }
        return fileDTOList.stream().map(this::dtoToDomain).collect(Collectors.toList());
    }


}
