package com.example.resapi.courses.application.mapper;


import com.example.resapi.courses.application.dto.CourseDTO;
import com.example.resapi.courses.application.dto.PublicationDTO;
import com.example.resapi.courses.domain.entity.CourseDomain;
import com.example.resapi.courses.domain.entity.PublicationDomain;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class PublicationMapperApplication implements Serializable {
    private static PublicationMapperApplication instance;

    private PublicationMapperApplication(){}

    public static PublicationMapperApplication getInstance(){
        if(instance==null){
            instance = new PublicationMapperApplication();
        }
        return instance;
    }

    public PublicationDTO domainToDto(PublicationDomain publicationDomain) {
        if(publicationDomain == null){
            return null;
        }
        CourseDTO courseDTO = CourseMapperApplication.getInstance().domainToDto(publicationDomain.getCourse());
        return PublicationDTO.builder()
                .id(publicationDomain.getId())
                .title(publicationDomain.getTitle())
                .description(publicationDomain.getDescription())
                .courseDTO(courseDTO)
                .courseName(courseDTO.getName())
                .build();
    }

    public PublicationDomain dtoToDomain(PublicationDTO publicationDTO){
        if(publicationDTO == null){
            return null;
        }
        CourseDomain courseDomain = CourseMapperApplication.getInstance().dtoToDomain(publicationDTO.getCourseDTO());

        return PublicationDomain.builder()
                .id(publicationDTO.getId())
                .title(publicationDTO.getTitle())
                .description(publicationDTO.getDescription())
                .course(courseDomain)
                .build();
    }

    public List<PublicationDTO> listDomainToListDto(List<PublicationDomain> publicationDomainList){
        if(publicationDomainList==null){
            return Collections.emptyList();
        }
        return publicationDomainList.stream().map(this::domainToDto).collect(Collectors.toList());
    }
    public List<PublicationDomain> listDtoToDomainList(List<PublicationDTO> publicationDTOList){
        if(publicationDTOList ==null){
            return Collections.emptyList();
        }
        return publicationDTOList.stream().map(this::dtoToDomain).collect(Collectors.toList());
    }

}


