package com.example.resapi.courses.application.publication.find;

import com.example.resapi.courses.application.dto.CourseDTO;
import com.example.resapi.courses.application.dto.PublicationDTO;
import com.example.resapi.courses.application.mapper.CourseMapperApplication;
import com.example.resapi.courses.application.mapper.PublicationMapperApplication;
import com.example.resapi.courses.application.repository.PublicationRepository;
import com.example.resapi.courses.domain.entity.PublicationDomain;
import com.example.resapi.shared.exceptions.TfgException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class FindPublicationImp implements FindPublication {

    private final PublicationRepository publicationRepository;

    @Autowired
    public FindPublicationImp(PublicationRepository publicationRepository) {
        this.publicationRepository = publicationRepository;
    }

    @Override
    public PublicationDTO findByTitle(String title) throws TfgException {
        PublicationDomain publicationDomain = PublicationMapperApplication.getInstance().dtoToDomain(publicationRepository.findByTitle(title));
        if(publicationDomain == null){
            return null;
        }
        CourseDTO courseDTO = CourseMapperApplication.getInstance().domainToDto(publicationDomain.getCourse());
        return PublicationDTO.builder()
                        .id(publicationDomain.getId())
                        .title(publicationDomain.getTitle())
                        .description(publicationDomain.getDescription())
                        .courseDTO(courseDTO)
                        .build();
    }
}
