package com.example.resapi.courses.infrastructure.controller_inputAdapter;

import com.example.resapi.courses.application.dto.CategoriesDTO;
import com.example.resapi.courses.infrastructure.controller_inputPort.CategoriesInputPort;
import com.example.resapi.courses.infrastructure.repository_outputAdapter.SQL.SqlCategoriesRepositoryImp;
import com.example.resapi.shared.exceptions.TfgException;
import com.example.resapi.shared.valueObject.ResponseDTO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/categories")
@CrossOrigin
@Slf4j
public class CategoriesController implements CategoriesInputPort {
    SqlCategoriesRepositoryImp categoriesRepository;

    @Autowired
    public CategoriesController(SqlCategoriesRepositoryImp categoriesRepository) {
        this.categoriesRepository = categoriesRepository;
    }
    @Override
    @GetMapping("/get/{categoriesId}")
    public ResponseDTO<CategoriesDTO> getCategoriesById(@PathVariable("categoriesId") Integer id){
        log.info("getRolById");
        ResponseDTO<CategoriesDTO> responseDTO = new ResponseDTO<>();
        try {
            CategoriesDTO categoriesDTO =this.categoriesRepository.getById(id);
            responseDTO.setData(categoriesDTO);
            responseDTO.addInfo("GET_CATEGORIES_ID_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }
    @Override
    @GetMapping("/find/{categoriesName}")
    public ResponseDTO<CategoriesDTO> getCategoriesByName(@PathVariable("categoriesName") String name){
        log.info("getRolById");
        ResponseDTO<CategoriesDTO> responseDTO = new ResponseDTO<>();
        try {
            CategoriesDTO categoriesDTO =this.categoriesRepository.findByName(name);
            responseDTO.setData(categoriesDTO);
            responseDTO.addInfo("GET_CATEGORIES_BY_NAME_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }



    @Override
    @GetMapping("/getAll")
    public ResponseDTO<List<CategoriesDTO>> getAll(){
        ResponseDTO<List<CategoriesDTO>> responseDTO = new ResponseDTO<>();
        try {
            List<CategoriesDTO> categoriesDTOList =this.categoriesRepository.getAll();
            responseDTO.setData(categoriesDTOList);
            responseDTO.addInfo("GET_ALL_CATEGORIES_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }
    @Override
    @PostMapping("/create")
    public ResponseDTO<CategoriesDTO> createCategories(@RequestBody CategoriesDTO categoriesDTO){
        ResponseDTO<CategoriesDTO> responseDTO = new ResponseDTO<>();
        try {
            CategoriesDTO categoriesDTO1 = this.categoriesRepository.create(categoriesDTO);
            responseDTO.setData(categoriesDTO1);
            responseDTO.addInfo("CREATE_CATEGORIES_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }
    @Override
    @DeleteMapping("/delete/{courseId}")
    public ResponseDTO<CategoriesDTO> deleteCategoriesByCourseId(@PathVariable("courseId") Integer id){
        ResponseDTO<CategoriesDTO> responseDTO = new ResponseDTO<>();
        try {
            CategoriesDTO categoriesDTO = this.categoriesRepository.delete(id);
            responseDTO.setData(categoriesDTO);
            responseDTO.addInfo("DELETE_CATEGORIES_BY_COURSE_ID_OK");
        }catch(
                TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }
    @Override
    @DeleteMapping("/delete/name/{courseName}")
    public ResponseDTO<CategoriesDTO> deleteCategoriesByCourseName (@PathVariable("courseName") String name){
        ResponseDTO<CategoriesDTO> responseDTO = new ResponseDTO<>();
        try {
            CategoriesDTO categoriesDTO = this.categoriesRepository.delete(name);
            responseDTO.setData(categoriesDTO);
            responseDTO.addInfo("DELETED_COURSE_BY_COURSE_NAME_OK");
        }catch(
                TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

}
