package com.example.resapi.courses.infrastructure.controller_inputAdapter;

import com.example.resapi.courses.application.chatMessage.create.CreateChatMessageImp;
import com.example.resapi.courses.application.chatMessage.delete.DeleteChatMessageImp;
import com.example.resapi.courses.application.chatMessage.find.FindChatMessageImp;
import com.example.resapi.courses.application.chatMessage.get.GetChatMessageByIdImp;
import com.example.resapi.courses.application.dto.ChatMessageDTO;
import com.example.resapi.courses.application.chatMessage.update.UpdateChatMessageImp;
import com.example.resapi.courses.infrastructure.controller_inputPort.ChatMessageInputPort;
import com.example.resapi.shared.exceptions.TfgException;
import com.example.resapi.shared.valueObject.ResponseDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.List;


@RestController
@RequestMapping("/chatmessage")
@CrossOrigin
public class ChatMessageController implements ChatMessageInputPort {
    private final CreateChatMessageImp createNotificationImp;
    private final DeleteChatMessageImp deleteNotificationImp;
    private final FindChatMessageImp findNotificationImp;
    private final GetChatMessageByIdImp getChatMessageByIdImp;
    private final UpdateChatMessageImp updateNotificationImp;

    @Autowired
    public ChatMessageController(CreateChatMessageImp createNotificationImp,
                                 DeleteChatMessageImp deleteNotificationImp,
                                 FindChatMessageImp findNotificationImp,
                                 GetChatMessageByIdImp getChatMessageByIdImp,
                                 UpdateChatMessageImp updateNotificationImp) {
        this.createNotificationImp = createNotificationImp;
        this.deleteNotificationImp = deleteNotificationImp;
        this.findNotificationImp = findNotificationImp;
        this.getChatMessageByIdImp = getChatMessageByIdImp;
        this.updateNotificationImp = updateNotificationImp;
    }

    @Override
    @GetMapping("/get/{chatmessageId}")
    public ResponseDTO<ChatMessageDTO> getChatMessageById(@PathVariable("chatmessageId") Integer notificationId){
        ResponseDTO<ChatMessageDTO> responseDTO = new ResponseDTO<>();
        try {
            ChatMessageDTO chatMessageDTO =this.getChatMessageByIdImp.getChatMessageByID(notificationId);
            responseDTO.setData(chatMessageDTO);
            responseDTO.addInfo("GET_CHAT_MESSAGE_ID_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @GetMapping("/getAll")
    public ResponseDTO<List<ChatMessageDTO>> getAll(){
        ResponseDTO<List<ChatMessageDTO>> responseDTO = new ResponseDTO<>();
        try {
            List<ChatMessageDTO> chatMessageDTOList =this.getChatMessageByIdImp.getAllNotification();
            responseDTO.setData(chatMessageDTOList);
            responseDTO.addInfo("GET_ALL_CHAT_MESSAGES_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @GetMapping("/find/{courseId}")
    public ResponseDTO<List<ChatMessageDTO>> findbycourse(@PathVariable("courseId") Integer courseId) {
        ResponseDTO<List<ChatMessageDTO>> responseDTO= new ResponseDTO<>();
        try {
            responseDTO.setData(this.findNotificationImp.findById_Course(courseId));
            responseDTO.addInfo("GET_ALL_CHAT_MESSAGES_OF_COURSE_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @PostMapping("/create")
    public ResponseDTO<ChatMessageDTO> create(@RequestBody ChatMessageDTO chatMessageDTO) {
        ResponseDTO<ChatMessageDTO> responseDTO=new ResponseDTO<>();
        System.out.println(LocalDateTime.now());
        chatMessageDTO.setTime(LocalDateTime.now());
        try {
            responseDTO.setData(this.createNotificationImp.create(chatMessageDTO));
            responseDTO.addInfo("CREATE_CHAT_MESSAGES_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @DeleteMapping("/delete/{chatMessageId}")
    public ResponseDTO<ChatMessageDTO> deleteChatMessageByIdMessage(@PathVariable("chatMessageId") Integer chatMessageId) {
        ResponseDTO<ChatMessageDTO> responseDTO=new ResponseDTO<>();
        try {
            responseDTO.setData(this.deleteNotificationImp.delete(chatMessageId));
            responseDTO.addInfo("DELETE_CHAT_MESSAGES__OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

}
