package com.example.resapi.courses.infrastructure.controller_inputAdapter;

import com.example.resapi.courses.application.course.create.CreateCourseImp;
import com.example.resapi.courses.application.course.delete.DeleteCourseImp;
import com.example.resapi.courses.application.course.find.FindCourseImp;
import com.example.resapi.courses.application.course.find.FindCousePublicationsImp;
import com.example.resapi.courses.application.course.get.GetCourseByIdImp;
import com.example.resapi.courses.application.course.update.UpdateCourseImp;
import com.example.resapi.courses.application.dto.CategoriesDTO;
import com.example.resapi.courses.application.dto.CourseDTO;
import com.example.resapi.courses.application.dto.PublicationDTO;
import com.example.resapi.courses.infrastructure.controller_inputPort.CourseInputPort;
import com.example.resapi.courses.infrastructure.repository_outputAdapter.SQL.SqlCategoriesRepositoryImp;
import com.example.resapi.shared.exceptions.TfgException;
import com.example.resapi.shared.valueObject.ResponseDTO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;


@RestController
@RequestMapping("/course")
@CrossOrigin
@Slf4j
public class CourseController implements CourseInputPort {
    private final CreateCourseImp createCourseImp;
    private final DeleteCourseImp deleteCourseImp;
    private final FindCourseImp findCourseImp;
    private final GetCourseByIdImp getCourseByIdImp;
    private final SqlCategoriesRepositoryImp categoriesRepository;
    private final UpdateCourseImp updateCourseImp;
    private final FindCousePublicationsImp findCousePublicationsImp;

    @Autowired
    public CourseController(CreateCourseImp createCourseImp,
                            DeleteCourseImp deleteCourseImp,
                            FindCourseImp findCourseImp,
                            GetCourseByIdImp getCourseByIdImp,
                            UpdateCourseImp updateCourseImp,
                            SqlCategoriesRepositoryImp categoriesRepository,
                            FindCousePublicationsImp findCousePublicationsImp) {
        this.createCourseImp = createCourseImp;
        this.deleteCourseImp = deleteCourseImp;
        this.findCourseImp = findCourseImp;
        this.getCourseByIdImp = getCourseByIdImp;
        this.updateCourseImp = updateCourseImp;
        this.categoriesRepository=categoriesRepository;
        this.findCousePublicationsImp = findCousePublicationsImp;
    }


    @Override
    @GetMapping("/get/{courseId}")
    public ResponseDTO<CourseDTO> getCourseById(@PathVariable("courseId") Integer rolId){
        log.info("getRolById");
        ResponseDTO<CourseDTO> responseDTO = new ResponseDTO<>();
        try {
            CourseDTO courseDTO=this.getCourseByIdImp.getCourseByID(rolId);
            responseDTO.setData(courseDTO);
            responseDTO.addInfo("GET_COURSE_BY_ROL_ID_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @GetMapping("/find/{courseName}")
    public ResponseDTO<CourseDTO> getCourseByName(@PathVariable("courseName") String name){
        log.info("getRolById");
        ResponseDTO<CourseDTO> responseDTO = new ResponseDTO<>();
        try {
            CourseDTO courseDTO=this.getCourseByIdImp.findByName(name);
            responseDTO.setData(courseDTO);
            responseDTO.addInfo("GET_COURSE_NAME_BY_ROL_ID_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @GetMapping("/publications/{courseName}")
    public ResponseDTO<List<PublicationDTO>> getCoursePublicationsByCourseName(@PathVariable("courseName") String name){
        log.info("getPublications");
        ResponseDTO<List<PublicationDTO>> responseDTO = new ResponseDTO<>();
        try {
            List<PublicationDTO> publicationDTOList= this.findCousePublicationsImp.publicationList(name);
            responseDTO.setData(publicationDTOList);
            responseDTO.addInfo("COURSE_GET_PUBLICATIONS_BY_COURSE_NAME_OK");
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @GetMapping("/getAll")
    public ResponseDTO<List<CourseDTO>> getAll(){
        ResponseDTO<List<CourseDTO>> responseDTO = new ResponseDTO<>();
        try {
            List<CourseDTO> lista= this.getCourseByIdImp.getAllCourses();
            responseDTO.setData(lista);
            responseDTO.addInfo("COURSE_GET_ALL_OK");
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @PostMapping("/create")
    public ResponseDTO<CourseDTO> createCourse (@RequestBody CourseDTO course){
        ResponseDTO<CourseDTO> responseDTO = new ResponseDTO<>();
        try {
            CourseDTO courseDTO=this.getCourseByIdImp.create(course);
            responseDTO.setData(courseDTO);
            responseDTO.addInfo("COURSE_CREATED_OK");
        }catch(
                TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @DeleteMapping("/delete/{courseId}")
    public ResponseDTO<CourseDTO> deleteCourse (@PathVariable("courseId") Integer id){
        ResponseDTO<CourseDTO> responseDTO = new ResponseDTO<>();
        try {
            CourseDTO courseDTO=this.getCourseByIdImp.delete(id);
            responseDTO.setData(courseDTO);
            responseDTO.addInfo("COURSE_DELETED_OK");
        }catch(
                TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @DeleteMapping("/delete/name/{courseName}")
    public ResponseDTO<CourseDTO> deleteCourseByName (@PathVariable("courseName") String name){
        ResponseDTO<CourseDTO> responseDTO = new ResponseDTO<>();
        try {
            CourseDTO courseDTO=this.getCourseByIdImp.delete(name);
            responseDTO.setData(courseDTO);
            responseDTO.addInfo("COURSE_DELETED_OK");
        }catch(
                TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @PostMapping("/categoria/{courseName}/{categoriaName}")
    public ResponseDTO<CourseDTO> addCategoryToCourse (@PathVariable("courseName") String cursoName, @PathVariable("categoriaName") String categoName ){
        ResponseDTO<CourseDTO> responseDTO = new ResponseDTO<>();
        try {
            CategoriesDTO categoriesDTO =categoriesRepository.findByName(categoName);
            CourseDTO courseDTO=getCourseByIdImp.findByName(cursoName);
            CourseDTO course1=getCourseByIdImp.asignarCatego(courseDTO, categoriesDTO);
            responseDTO.setData(course1);
            responseDTO.addInfo("Asignada_Categorie_OK");
        }catch(
                TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @PostMapping("/quitarcategoria/{courseName}/{categoriaName}")
    public ResponseDTO<CourseDTO> deleteCategoryToCourse (@PathVariable("courseName") String cursoName, @PathVariable("categoriaName") String categoName ){
        ResponseDTO<CourseDTO> responseDTO = new ResponseDTO<>();
        try {
            CategoriesDTO categoriesDTO =categoriesRepository.findByName(categoName);
            CourseDTO courseDTO=getCourseByIdImp.findByName(cursoName);
            CourseDTO course1=getCourseByIdImp.desasignarCatego(courseDTO, categoriesDTO);
            responseDTO.setData(course1);
            responseDTO.addInfo("DESASAGNAR_CATEGOY_FROM_COURSE_OK");
        }catch(
                TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @PutMapping("update")
    public ResponseDTO<CourseDTO> updateCourse (@RequestBody CourseDTO course) {
        ResponseDTO<CourseDTO> responseDTO = new ResponseDTO<>();
        try {
            responseDTO.setData(this.updateCourseImp.update(course));
            responseDTO.addInfo("COURSE_UPDATED_OK");
        }catch(
                TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @GetMapping("coursesbyprofe/{courseId}")
    public ResponseDTO<List<CourseDTO>> getAllCursosProfe(@PathVariable("courseId") Integer id){
        log.info("cursosProfe: "+id);
        ResponseDTO<List<CourseDTO>> responseDTO = new ResponseDTO<>();
        try {
            List<CourseDTO> listCourse = new ArrayList<>();
            List<CourseDTO> courseList= this.getCourseByIdImp.getAllCoursesbyuser();
            for (CourseDTO curso:courseList) {
                if (curso.getList_id_professors().contains(id))
                    listCourse.add(curso);
            }
            responseDTO.setData(listCourse);
            responseDTO.addInfo("GET_ALL_COURSE_BY_ID_PROF_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @GetMapping("coursesbyalumn/{courseId}")
    public ResponseDTO<List<CourseDTO>> cursosStudent(@PathVariable("courseId") Integer id){
        log.info("cursosAlum: "+id);
        ResponseDTO<List<CourseDTO>> responseDTO = new ResponseDTO<>();

        try {
            List<CourseDTO> listCourse = new ArrayList<>();
            List<CourseDTO> courseList= this.getCourseByIdImp.getAllCoursesbyuser();
            for (CourseDTO curso:courseList) {
                if (curso.getList_id_students().contains(id))
                    listCourse.add(curso);
            }
            responseDTO.setData(listCourse);
            responseDTO.addInfo("GET_ALL_COURSE_BY_ID_ALUMNO_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @GetMapping("coursesNoId/{userId}")
    public ResponseDTO<List<CourseDTO>> cursosNoId(@PathVariable("userId") Integer id){
        log.info("coursesNoId: "+id);
        ResponseDTO<List<CourseDTO>> responseDTO = new ResponseDTO<>();

        try {
            List<CourseDTO> listCourse = new ArrayList<>();
            List<CourseDTO> courseList= this.getCourseByIdImp.getAllCoursesbyuser();
            for (CourseDTO curso:courseList) {
                if (!curso.getList_id_students().contains(id) && !curso.getList_id_professors().contains(id))
                    listCourse.add(curso);
            }
            responseDTO.setData(listCourse);
            responseDTO.addInfo("GET_ALL_COURSE_NO_ID_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @GetMapping("/recommended")
    public ResponseDTO<List<CourseDTO>> recomendados() {
        ResponseDTO<List<CourseDTO>> responseDTO = new ResponseDTO<>();
        try {
            List<CourseDTO> lista= this.getCourseByIdImp.getAllCourses();
            int tamanyo=lista.size();
            if (tamanyo>8)
                lista=lista.subList(0,7);
            responseDTO.setData(lista);
            responseDTO.addInfo("GET_COURSE_RECOMMENDED_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @GetMapping ("/estudiante/{IdCourse}/{IdAlumno}")
    public ResponseDTO<CourseDTO> asignarestudiante(@PathVariable("IdCourse") Integer idCourse, @PathVariable("IdAlumno") Integer idAlumno) {
        ResponseDTO<CourseDTO> responseDTO = new ResponseDTO<>();
        try {
            CourseDTO courseDTO=this.getCourseByIdImp.getCourseByID(idCourse);
            Set<Integer> estudiantes=courseDTO.getList_id_students();
            estudiantes.add(idAlumno);
            courseDTO.setList_id_students(estudiantes);
            responseDTO.setData(this.updateCourseImp.update(courseDTO));
            responseDTO.addInfo("COURSE_ADD_STUDENT_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;

    }

    @Override
    @GetMapping ("/profesor/{IdCourse}/{IdProfe}")
    public ResponseDTO<CourseDTO> asignarprofesor(@PathVariable("IdCourse") Integer idCourse, @PathVariable("IdProfe") Integer IdProfe) {
        ResponseDTO<CourseDTO> responseDTO = new ResponseDTO<>();
        try {
            CourseDTO courseDTO=this.getCourseByIdImp.getCourseByID(idCourse);
            Set<Integer> profesores=courseDTO.getList_id_professors();
            profesores.add(IdProfe);
            courseDTO.setList_id_professors(profesores);
            responseDTO.setData(this.updateCourseImp.update(courseDTO));
            responseDTO.addInfo("COURSE_ADD_PROFESSOR_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;

    }
}
