package com.example.resapi.courses.infrastructure.controller_inputAdapter;

import com.example.resapi.courses.application.dto.FileDTO;
import com.example.resapi.courses.application.file.create.CreateFileImp;
import com.example.resapi.courses.application.file.delete.DeleteFileImp;
import com.example.resapi.courses.application.file.find.FindFileImp;
import com.example.resapi.courses.application.file.get.GetFileByIdImp;
import com.example.resapi.courses.application.file.update.UpdateFileImp;
import com.example.resapi.courses.infrastructure.controller_inputPort.FileInputPort;
import com.example.resapi.shared.exceptions.TfgException;
import com.example.resapi.shared.valueObject.ResponseDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@RestController
@RequestMapping("/file")
@CrossOrigin
public class FileController implements FileInputPort {
    private final CreateFileImp createFileImp;
    private final DeleteFileImp deleteFileImp;
    private final FindFileImp findFileImp;
    private final GetFileByIdImp getFileByIdImp;
    private final UpdateFileImp updateFileImp;

    @Autowired
    public FileController(CreateFileImp createFileImp,
                          DeleteFileImp deleteFileImp,
                          FindFileImp findFileImp,
                          GetFileByIdImp getFileByIdImp,
                          UpdateFileImp updateFileImp) {
        this.createFileImp = createFileImp;
        this.deleteFileImp = deleteFileImp;
        this.findFileImp = findFileImp;
        this.getFileByIdImp = getFileByIdImp;
        this.updateFileImp = updateFileImp;
    }

    @Override
    @GetMapping("/get/{fileId}")
    public ResponseDTO<FileDTO> getFileById(@PathVariable("fileId") Integer fileId){
        ResponseDTO<FileDTO> responseDTO = new ResponseDTO<>();
        try {
            responseDTO.setData(this.getFileByIdImp.getFileByID(fileId));
            responseDTO.addInfo("GET_FILE_ID_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @GetMapping("/getAll")
    public ResponseDTO<List<FileDTO>> getAll(){
        ResponseDTO<List<FileDTO>> responseDTO = new ResponseDTO<>();
        try {
            responseDTO.setData(getFileByIdImp.getAllFile());
            responseDTO.addInfo("GET_ALL_FILES_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @PostMapping("/create")
    public ResponseDTO<FileDTO> createFile (@RequestBody FileDTO file) {
        ResponseDTO<FileDTO> responseDTO = new ResponseDTO<>();
        try {
            responseDTO.setData(this.createFileImp.create(file));
            responseDTO.addInfo("FILE_CREATED_OK");
        }catch(
                TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }

        return responseDTO;
    }

    @Override
    @DeleteMapping("/delete/{fileId}")
    public ResponseDTO<FileDTO> deleteFile(@PathVariable("fileId") Integer fileId) {
        ResponseDTO<FileDTO> responseDTO = new ResponseDTO<>();
        try {
            responseDTO.setData(this.deleteFileImp.delete(fileId));
            responseDTO.addInfo("FILE_DELETE_OK");
        }catch(
                TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }

        return responseDTO;
    }


}
