package com.example.resapi.courses.infrastructure.controller_inputAdapter;

import com.example.resapi.courses.application.dto.FileDTO;
import com.example.resapi.courses.application.dto.PublicationDTO;
import com.example.resapi.courses.application.publication.create.CreatePublicationImp;
import com.example.resapi.courses.application.publication.delete.DeletePublicationImp;
import com.example.resapi.courses.application.publication.find.FindPublicationImp;
import com.example.resapi.courses.application.publication.get.GetPublicationByIdImp;
import com.example.resapi.courses.application.publication.update.UpdatePublicationImp;
import com.example.resapi.courses.infrastructure.controller_inputPort.PublicationInputPort;
import com.example.resapi.shared.exceptions.TfgException;
import com.example.resapi.shared.valueObject.ResponseDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@RestController
@RequestMapping("/publication")
@CrossOrigin
public class PublicationController implements PublicationInputPort {
    private final CreatePublicationImp createPublicationImp;
    private final DeletePublicationImp deletePublicationImp;
    private final FindPublicationImp findPublicationImp;
    private final GetPublicationByIdImp getPublicationByIdImp;
    private final UpdatePublicationImp updatePublicationImp;

    @Autowired
    public PublicationController(CreatePublicationImp createPublicationImp,
                                 DeletePublicationImp deletePublicationImp,
                                 FindPublicationImp findPublicationImp,
                                 GetPublicationByIdImp getPublicationByIdImp,
                                 UpdatePublicationImp updatePublicationImp) {
        this.createPublicationImp = createPublicationImp;
        this.deletePublicationImp = deletePublicationImp;
        this.findPublicationImp = findPublicationImp;
        this.getPublicationByIdImp = getPublicationByIdImp;
        this.updatePublicationImp = updatePublicationImp;
    }

    @Override
    @GetMapping("/get/{publicationId}")
    public ResponseDTO<PublicationDTO> getPublicationById(@PathVariable("publicationId") Integer publicationId){
        ResponseDTO<PublicationDTO> responseDTO = new ResponseDTO<>();
        try {
            responseDTO.setData(this.getPublicationByIdImp.getPublicationByID(publicationId));
            responseDTO.addInfo("GET_PUBLICATION_BY_ID_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @GetMapping("/getAll")
    public ResponseDTO<List<PublicationDTO>> getAll(){
        ResponseDTO<List<PublicationDTO>> responseDTO = new ResponseDTO<>();
        try {
            responseDTO.setData(this.getPublicationByIdImp.getAllPublication());
            responseDTO.addInfo("PUBLICATION_GET_ALL_OK");
        }catch(
                TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @PostMapping("/create")
    public ResponseDTO<PublicationDTO> createPublicacion(@RequestBody PublicationDTO publicationDTO) {
        ResponseDTO<PublicationDTO> responseDTO = new ResponseDTO<>();
        try {
            responseDTO.setData(this.createPublicationImp.create(publicationDTO));
            responseDTO.addInfo("PUBLICATION_CREATED_OK");
        }catch(
                TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @Override
    @PutMapping("/update")
    public ResponseDTO<PublicationDTO> updatePublicacion(@RequestBody PublicationDTO publication) {
        ResponseDTO<PublicationDTO> responseDTO = new ResponseDTO<>();
        try {
            responseDTO.setData(this.updatePublicationImp.update(publication));
            responseDTO.addInfo("PUBLICATION_UPDATED_OK");
        }catch(
                TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @GetMapping("/find/{title}")
    public ResponseDTO<PublicationDTO> getPublicationByTitle(@PathVariable("title") String title){
        ResponseDTO<PublicationDTO> responseDTO = new ResponseDTO<>();
        try {
            responseDTO.setData(this.findPublicationImp.findByTitle(title));
            responseDTO.addInfo("FIND_PUBLICATION_BY_TITLE_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }

    @GetMapping("/find/files/{publiId}")
    public ResponseDTO<List<FileDTO>> getAllFilesByPublicationId(@PathVariable("publiId") Integer publicationId){
        ResponseDTO<List<FileDTO>> responseDTO= new ResponseDTO<>();
        try {
            PublicationDTO publicationDTO=this.getPublicationByIdImp.getPublicationByID(publicationId);
            responseDTO.setData(publicationDTO.getFile());
            responseDTO.addInfo("FIND_FILES_BY_PUBLICATIONID_OK");
        }catch(TfgException e){
            responseDTO.setErrorMessages(e.getErrorMessages());
        }catch (Exception e){
            responseDTO.addError(e.getMessage());
        }
        return responseDTO;
    }
}
