package com.example.resapi.courses.infrastructure.mapper;

import com.example.resapi.courses.application.dto.CategoriesDTO;
import com.example.resapi.courses.application.dto.CourseDTO;
import com.example.resapi.courses.application.dto.PublicationDTO;
import com.example.resapi.courses.infrastructure.entity.CategoriesEntity;
import com.example.resapi.courses.infrastructure.entity.CourseEntity;
import org.json.JSONArray;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.*;
import java.util.stream.Collectors;
@Service
public class CourseMapper implements Serializable {
    private static CourseMapper instance;

    private CourseMapper(){}

    public static CourseMapper getInstance(){
        if(instance==null){
            instance = new CourseMapper();
        }
        return instance;
    }

    private Set<Integer> StrToSet (String str) {
        Set<Integer> set = new HashSet<>();
        JSONArray jsonArray = new JSONArray(str);
        for (Object i:jsonArray){
            set.add((Integer)i);
        }
        return set;
    }

    private String SetToStr (Set<Integer> set) {
        JSONArray jsonArray = new JSONArray(set);
        //String resu = "["+CDL.rowToString(jsonArray).substring(0,CDL.rowToString(jsonArray).length()-1)+"]";
        return jsonArray.toString();
        //String resu3 = jsonArray.toString(0);
    }

    public CourseDTO entityToDto(CourseEntity courseEntity) {
        if(courseEntity == null){
            return null;
        }
        List<CategoriesDTO> categoriesDTOList = CategoriesMapper.getInstance().listEntityToDtoList(courseEntity.getList_Categories());
        List<PublicationDTO> publicationDTOList = PublicationMapper.getInstance().listEntityToDtoList(courseEntity.getPublications());
        return CourseDTO.builder()
                .id(courseEntity.getId())
                .name(courseEntity.getName())
                .price(courseEntity.getPrice())
                .list_Categories(categoriesDTOList)
                .list_id_professors(this.StrToSet(courseEntity.getList_id_professors()))
                .list_id_students(this.StrToSet(courseEntity.getList_id_students()))
                .list_id_publications(publicationDTOList) //TODO
                .build();
    }
    public CourseEntity dtoToEntity(CourseDTO courseDTO){
        if(courseDTO == null){
            return null;
        }
        List<CategoriesEntity> categoriesEntityList = CategoriesMapper.getInstance().listDtoToEntityList(courseDTO.getList_Categories());
        //List<PublicationEntity> publicationEntityList = PublicationMapper.getInstance().listDtoToEntityList(courseDTO.getList_id_publications());

        return CourseEntity.builder()
            .id(courseDTO.getId())
            .name(courseDTO.getName())
            .price(courseDTO.getPrice())
            .list_Categories(categoriesEntityList)
            .list_id_professors(this.SetToStr(courseDTO.getList_id_professors()))
            .list_id_students(this.SetToStr(courseDTO.getList_id_students()))
            //.publications(publicationEntityList) //TODO
            .build();
    }

    public CourseDTO bothToDTO(CourseDTO courseDTO, CourseDTO course){
        if(courseDTO == null){
            return null;
        }
        if (courseDTO.getName()!=null && !Objects.equals(courseDTO.getName(), ""))
            course.setName(courseDTO.getName());
        if (courseDTO.getPrice()!=null)
            course.setPrice(courseDTO.getPrice());
        if (courseDTO.getList_Categories()!=null && !courseDTO.getList_Categories().isEmpty())
            course.setList_Categories(courseDTO.getList_Categories());
        if (courseDTO.getList_id_professors()!=null && !courseDTO.getList_id_professors().isEmpty())
            course.setList_id_professors(courseDTO.getList_id_professors());
        if (courseDTO.getList_id_students()!=null && !courseDTO.getList_id_students().isEmpty())
            course.setList_id_students(courseDTO.getList_id_students());
        return course;
    }


    public List<CourseEntity> listDtoToEntityList(List<CourseDTO> courseDTOList){
        if(courseDTOList==null){
            return Collections.emptyList();
        }
        return courseDTOList.stream().map(this::dtoToEntity).collect(Collectors.toList());
    }

    public List<CourseDTO> listEntityToDtoList(List<CourseEntity> courseEntityList){
        if(courseEntityList==null){
            return Collections.emptyList();
        }
        return courseEntityList.stream().map(this::entityToDto).collect(Collectors.toList());
    }

}
