package com.example.resapi.courses.infrastructure.mapper;


import com.example.resapi.courses.application.dto.CourseDTO;
import com.example.resapi.courses.application.dto.FileDTO;
import com.example.resapi.courses.application.dto.PublicationDTO;
import com.example.resapi.courses.infrastructure.entity.CourseEntity;
import com.example.resapi.courses.infrastructure.entity.FileEntity;
import com.example.resapi.courses.infrastructure.entity.PublicationEntity;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class PublicationMapper implements Serializable {
    private static PublicationMapper instance;

    private PublicationMapper(){}

    public static PublicationMapper getInstance(){
        if(instance==null){
            instance = new PublicationMapper();
        }
        return instance;
    }


    public PublicationDTO entityToDto(PublicationEntity publicationEntity) {
        if(publicationEntity == null){
            return null;
        }
        //CourseEntity courseEntity = publicationEntity.getCourse();
        //CourseDTO courseDTO = CourseMapper.getInstance().entityToDto(courseEntity);
        List<FileDTO> fileDTOList = FileMapper.getInstance().listEntityToDtoList(publicationEntity.getFile());

        return PublicationDTO.builder()
                .id(publicationEntity.getId())
                .title(publicationEntity.getTitle())
                .description(publicationEntity.getDescription())
                //.courseDTO(courseDTO)
                .courseName(publicationEntity.getCourse()==null?null:publicationEntity.getCourse().getName())
                .file(fileDTOList)
                .build();
    }

    public PublicationEntity dtoToEntity(PublicationDTO publicationDTO){
        if(publicationDTO == null){
            return null;
        }
        CourseDTO courseDTO = publicationDTO.getCourseDTO();
        CourseEntity courseEntity = CourseMapper.getInstance().dtoToEntity(courseDTO);
        List<FileEntity> fileEntityList = FileMapper.getInstance().listDtoToEntityList(publicationDTO.getFile());

        return PublicationEntity.builder()
                .id(publicationDTO.getId())
                .title(publicationDTO.getTitle())
                .description(publicationDTO.getDescription())
                .course(courseEntity)
                .file(fileEntityList)
                .build();
    }

    public List<PublicationEntity> listDtoToEntityList(List<PublicationDTO> publicationDTOList){
        if(publicationDTOList ==null){
            return Collections.emptyList();
        }
        return publicationDTOList.stream().map(this::dtoToEntity).collect(Collectors.toList());
    }

    public List<PublicationDTO> listEntityToDtoList(List<PublicationEntity> publicationEntityList){
        if(publicationEntityList==null){
            return Collections.emptyList();
        }
        List<PublicationDTO> publicationDTOList = publicationEntityList.stream().map(this::entityToDto).collect(Collectors.toList());
        return publicationDTOList;
    }



}


