package com.example.resapi.courses.infrastructure.repository_outputAdapter.SQL;

import com.example.resapi.courses.application.dto.PublicationDTO;
import com.example.resapi.courses.application.mapper.PublicationMapperApplication;
import com.example.resapi.courses.application.repository.PublicationRepository;
import com.example.resapi.courses.infrastructure.entity.PublicationEntity;
import com.example.resapi.courses.infrastructure.mapper.PublicationMapper;
import com.example.resapi.courses.infrastructure.repository_outputPort.SQL.SqlPublicationRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class SqlPublicationRepositoryImp implements PublicationRepository {

    private final SqlPublicationRepository sqlPublicationRepository;
    private final PublicationMapper publicationMapper = PublicationMapper.getInstance();
    private final PublicationMapperApplication publicationMapperApplication = PublicationMapperApplication.getInstance();

    @Autowired
    public SqlPublicationRepositoryImp(SqlPublicationRepository sqlPublicationRepository) {
        this.sqlPublicationRepository = sqlPublicationRepository;
    }


    @Override
    public PublicationDTO create(PublicationDTO publicationDTO) {
        PublicationEntity publicationEntity = publicationMapper.dtoToEntity(publicationDTO);
        return publicationMapper.entityToDto(sqlPublicationRepository.save(publicationEntity));
    }

    @Override
    public PublicationDTO getById(Integer id) {
        return this.publicationMapper.entityToDto(this.sqlPublicationRepository.findById(id).orElse(null));
    }

    @Override
    public PublicationDTO findByTitle(String title) {

        return publicationMapper.entityToDto(this.sqlPublicationRepository.findByTitle(title).orElse(null));
    }

    @Override
    public PublicationDTO update(PublicationDTO notification, PublicationDTO original) {
        if (notification.getDescription()!=null)
            original.setDescription(notification.getDescription());
        if (notification.getTitle()!=null)
            original.setTitle(notification.getTitle());
        if (notification.getCourseDTO()!=null)
            original.setCourseDTO(notification.getCourseDTO());
        return PublicationMapper.getInstance().entityToDto(this.sqlPublicationRepository.save(PublicationMapper.getInstance().dtoToEntity(original)));
    }

    @Override
    public List<PublicationDTO> getAll() {
        List<PublicationEntity> publicationEntityList = this.sqlPublicationRepository.findAll();
        return this.publicationMapper.listEntityToDtoList(publicationEntityList);
    }
    @Override
    public PublicationDTO delete(PublicationDTO publicationDTO) {
        this.sqlPublicationRepository.delete(publicationMapper.dtoToEntity(publicationDTO));
        return publicationDTO;
    }
}
